//
//  ASRTRStyleView.swift
//  AIPlayRingtones
//
//  Created by mini on 2025/5/19.
//

class ASRTRStyleView: ASBaseView {
    
    var selectedCompleted:((ASGenerateStyleModel,Int)->Void)?

    //选择生成类类型组
    var generateStyleIndex:Int = 0{
        didSet{
            if generateStyleModels.count <= 0 { return }
            DispatchQueue.main.async {
                self.collectionView.selectItem(at: IndexPath(item: self.generateStyleIndex, section: 0), animated: false, scrollPosition: .left)
            }
        }
    }
    
    var generateStyleModels:[ASGenerateStyleModel] = []
    {
        didSet{
            collectionView.reloadData()
            self.generateStyleIndex = abs(generateStyleIndex)
        }
    }
    var generateStyleModel:ASGenerateStyleModel?{
        didSet{
            if let generateStyleModel = generateStyleModel {
                selectedCompleted?(generateStyleModel,generateStyleIndex)
            }
        }
    }

    private var collectionViewObserver: ASColObserver!
    lazy var layout: UICollectionViewFlowLayout = {
        let layout = UICollectionViewFlowLayout()
        layout.scrollDirection = .vertical
        layout.itemSize = kGetCollectionViewCellSize(sectionInset: UIEdgeInsets(top: 0, left: 16, bottom: 0, right: 16), itemSpacing: 16, originalSize: CGSize(width: 80, height: 104), cellRowNum: 4)
        layout.minimumInteritemSpacing = 8
        layout.minimumLineSpacing = 16
        return layout
    }()
    
    lazy var collectionView: UICollectionView = {
        let collectionView = UICollectionView(frame: .zero, collectionViewLayout: layout)
        collectionView.isScrollEnabled = false
        collectionView.delegate = self
        collectionView.dataSource = self
        collectionView.showsVerticalScrollIndicator = false
        collectionView.showsHorizontalScrollIndicator = false
        collectionView.backgroundColor = .clear
        collectionView.contentInset = UIEdgeInsets(top: 0, left: 16, bottom: 0, right: 16)
        collectionView.register(ASRTRStyleViewCell.self, forCellWithReuseIdentifier: ASRTRStyleViewCell.cellID)
        if #available(iOS 11.0, *) {
            collectionView.contentInsetAdjustmentBehavior = .never
        }
        return collectionView
    }()
    
    
    override func creatUI() {
        contentView.addSubview(collectionView)
        collectionView.snp.makeConstraints { make in
            make.edges.equalToSuperview()
            make.height.equalTo(0)
        }
    }

    override func dealThings() {

        // 监听collectionView 的 contentSize
        collectionViewObserver = ASColObserver(collectionView: collectionView)
        collectionViewObserver.onContentSizeChange = { [weak self] size in
            guard let self = self else { return }
            print("collectionViewObserver 内容大小变化: \(size)")
            self.collectionView.snp.updateConstraints { make in
                make.height.equalTo(size.height)
            }
        }
    }
}
extension ASRTRStyleView: UICollectionViewDataSource ,UICollectionViewDelegate,UICollectionViewDelegateFlowLayout {
    
    public func numberOfSections(in collectionView: UICollectionView) -> Int {
        return 1
    }
    
    public func collectionView(_ collectionView: UICollectionView, numberOfItemsInSection section: Int) -> Int {
        return generateStyleModels.count
    }
    
    public func collectionView(_ collectionView: UICollectionView, cellForItemAt indexPath: IndexPath) -> UICollectionViewCell {
        let cell = collectionView.dequeueReusableCell(withReuseIdentifier: ASRTRStyleViewCell.cellID, for: indexPath) as! ASRTRStyleViewCell
        if let model = generateStyleModels.safeAny(At: indexPath.item){
            cell.model = model
        }
        return cell
    }

    public func collectionView(_ collectionView: UICollectionView, didSelectItemAt indexPath: IndexPath) {
        generateStyleIndex = indexPath.item
        if let model = generateStyleModels.safeAny(At: generateStyleIndex) {
            generateStyleModel = model
        }
    }
    
}

class ASRTRStyleViewCell: ASBaseCollectionCell  {
    static let cellID = "ASRTRStyleViewCell"
    var model: ASGenerateStyleModel = ASGenerateStyleModel()  {
        didSet {
            proImageView.isHidden = !model.vip
            textLab.text = model.imageText.localized
            imageView.image = UIImage(named: model.imageName)
        }
    }
    
    override var isSelected: Bool{
        didSet{
            textLab.textColor = .white.withAlphaComponent( isSelected ? 1.0 : 0.7)
            boardImageView.isHidden = !isSelected
        }
    }

    lazy var boardImageView: UIImageView = {
        let boardImageView = UIImageView()//.createImageV(image: .ringTextSelectedBoard)
        boardImageView.isHidden = true
        
        DispatchQueue.main.async {
            boardImageView.addGradientBorder(colors: ["#FF96F5".uiColor,"#F4F1EA".uiColor,"#5F7CFF".uiColor],width: 1.5,radius:16*kDesignScale,startPoint: CGPoint(x: 0.5, y: 0),endPoint: CGPoint(x: 0.5, y: 1))
        }

        return boardImageView
    }()

    lazy var imageView: UIImageView = {
        let imageView = UIImageView()
        imageView.cornerRadius = 12
        return imageView
    }()
    
    lazy var proImageView:UIImageView = {
        let proImageView = UIImageView.createImageV(image: .PRO)
        return proImageView
    }()
    

    lazy var textLab: UILabel = {
        let textLab = UILabel.createLabel(font: .font(size:14,weight: .regular),textColor: .white.withAlphaComponent(0.6),textAlignment: .center)
        return textLab
    }()
    
    
    override func creatUI() {
        
        contentView.addSubview(boardImageView)
        boardImageView.snp.makeConstraints { make in
            make.width.height.equalTo(72.0*kDesignScale)
            make.top.equalTo(0)
            make.centerX.equalToSuperview()
        }
    
        contentView.addSubview(imageView)
        imageView.snp.makeConstraints { make in
            make.width.height.equalTo(64.0*kDesignScale)
            make.top.equalTo(4)
            make.centerX.equalToSuperview()
        }
        

        contentView.addSubview(textLab)
        textLab.snp.makeConstraints { make in
            make.top.equalTo(boardImageView.snp.bottom)
            make.leading.trailing.equalTo(0)
            make.bottom.equalTo(0)
        }
        
        imageView.addSubview(proImageView)
        proImageView.snp.makeConstraints { make in
            make.top.equalTo(6)
            make.trailing.equalTo(-8)
            make.width.equalTo(21)
            make.height.equalTo(10)
        }
 

    }
}
