//
//  APLaunchVC.swift
//  AIPlayRingtones
//
//  Created by 100Years on 2025/5/15.
//

import Alamofire
import UIKit

class ASLaunchVC: UIViewController {
    var dismissHandler: (() -> Void)?

    // 定时器
    private var timer: DispatchSourceTimer?
    // 闪屏页剩余显示时长
    #if DEBUG
    private var remindTimeInterval: TimeInterval = 2.0
    #else
        private var remindTimeInterval: TimeInterval = 2.0
    #endif

    override func viewDidLoad() {
        super.viewDidLoad()
        setupLaunchScreenView()
        addNotifiy()
        
        if AppDelegate.isFirstInstallApp() {//首次安装需要等待网络权限弹窗
            startTimer()
        }else {
            self.enterApp()
        }
    }

    func addNotifiy() {
        let networkManager = NetworkReachabilityManager.default
        // 开始监听网络状态
        networkManager?.startListening { status in
            switch status {
            case .reachable:
//                PurchaseManager.default.requestProducts()
                networkManager?.stopListening()
                break
            default:
//                PurchaseManager.default.requestProducts()
                break
            }
        }
    }

    func enterApp() {
        dismissHandler?()
    }


    
    private func startTimer() {
        if timer == nil {
            
            var maxTime:Int = 0 //200*10=2s 200 *10*3 =6s
            timer = DispatchSource.makeTimerSource(queue: DispatchQueue.global())
            timer?.schedule(deadline: .now(), repeating: .milliseconds(200))
            timer?.setEventHandler(handler: { [weak self] in
                // App活跃时，计时有效；如网络授权弹窗唤起时，计时失效
                DispatchQueue.main.async {
                    guard let self = self  else {
                        return
                    }
                    
                    guard UIApplication.shared.applicationState == .active
                    else {
                        logPrint("倒计时：!= active")
                        maxTime += 1
                        if maxTime > 30 {
                            logPrint("超过最大倒计时进入 app")
                            self.timer?.cancel()
                            self.timer = nil
                            self.enterApp()
                        }
                        
                        return
                    }
                    
                    self.remindTimeInterval -= 0.2
                    logPrint("倒计时：\(self.remindTimeInterval)")
                    if self.remindTimeInterval <= 0 {
                        self.timer?.cancel()
                        self.timer = nil
                        self.enterApp()
                    }
                }

            })
            timer?.resume()
        }else{
            self.enterApp()
        }
    }
    
    

    private func setupLaunchScreenView() {
        // 获取 LaunchScreen.storyboard 的视图控制器
        let storyboard = UIStoryboard(name: "LaunchScreen", bundle: nil)
        guard let launchVC = storyboard.instantiateInitialViewController() else {
            logPrint("无法加载 LaunchScreen 的初始视图控制器")
            return
        }

        // 获取 LaunchScreen 的视图
        guard let launchView = launchVC.view else {
            logPrint("无法获取 LaunchScreen 的视图")
            return
        }

        // 设置 launchView 的 frame 为当前视图的 bounds
        launchView.frame = UIScreen.main.bounds

        // 将 launchView 添加到 ASLaunchVC 的视图中
        view.addSubview(launchView)
    }
}

