//
//  APRingStyleVC.swift
//  AIPlayRingtones
//
//  Created by 100Years on 2025/5/16.
//

import ObjectMapper
class APRingStyleVC: ASBaseViewController {
    
    var selectedCompleted:((ASGenerateStyleModel,Int)->Void)?

    //选择生成类类型组
    var generateStyleIndex:Int = 0{
        didSet{
            if generateStyleModels.count <= 0 { return }
            DispatchQueue.main.async {
                self.collectionView.selectItem(at: IndexPath(item: self.generateStyleIndex, section: 0), animated: true, scrollPosition: .centeredVertically)
            }
        }
    }
    var generateStyleModel:ASGenerateStyleModel?{
        didSet{
            if let generateStyleModel = generateStyleModel {
                selectedCompleted?(generateStyleModel,generateStyleIndex)
            }
        }
    }
    lazy var generateStyleModels: [ASGenerateStyleModel] = {
        var ptpStyleModels = [ASGenerateStyleModel]()
        if let dataArray = Mapper<ASGenerateStyleModel>().mapArray(JSONfile: "ring_ttr_style.json"){
            ptpStyleModels = dataArray
            
            if let model = dataArray.safeAny(At: generateStyleIndex) {
                generateStyleModel = model //加上默认的选择
            }
        }
        
        return ptpStyleModels
    }()
    
    private var collectionViewObserver: ASColObserver!
    lazy var layout: UICollectionViewFlowLayout = {
        let layout = UICollectionViewFlowLayout()
        layout.scrollDirection = .vertical
        layout.itemSize = kGetCollectionViewCellSize(sectionInset: UIEdgeInsets(top: 0, left: 16, bottom: 0, right: 16), itemSpacing: 16, originalSize: CGSize(width: 104, height: 104), cellRowNum: 3)
        layout.minimumInteritemSpacing = 16
        layout.minimumLineSpacing = 16
        return layout
    }()
    
    lazy var collectionView: UICollectionView = {
        let collectionView = UICollectionView(frame: .zero, collectionViewLayout: layout)
        collectionView.isScrollEnabled = false
        collectionView.delegate = self
        collectionView.dataSource = self
        collectionView.showsVerticalScrollIndicator = false
        collectionView.showsHorizontalScrollIndicator = false
        collectionView.backgroundColor = .clear
        collectionView.contentInset = UIEdgeInsets(top: 0, left: 16, bottom: 0, right: 16)
        collectionView.register(APRingStyleCell.self, forCellWithReuseIdentifier: APRingStyleCell.cellID)
        if #available(iOS 11.0, *) {
            collectionView.contentInsetAdjustmentBehavior = .never
        }
        return collectionView
    }()
    
    override func createView() {
        setNavBarViewHidden(true)
        contentView.addSubview(collectionView)
        collectionView.snp.makeConstraints { make in
            make.edges.equalToSuperview()
            make.height.equalTo(k_ScreenHeight)
        }
    }

    override func dealThings() {

        // 监听collectionView 的 contentSize
        collectionViewObserver = ASColObserver(collectionView: collectionView)
        collectionViewObserver.onContentSizeChange = { [weak self] size in
            guard let self = self else { return }
            print("collectionViewObserver 内容大小变化: \(size)")
            collectionView.snp.updateConstraints { make in
                make.height.equalTo(size.height)
            }
        }
        
        collectionView.reloadData()
        self.generateStyleIndex = abs(generateStyleIndex)
    }
}

extension APRingStyleVC: UICollectionViewDataSource ,UICollectionViewDelegate,UICollectionViewDelegateFlowLayout {
    
    public func numberOfSections(in collectionView: UICollectionView) -> Int {
        return 1
    }
    
    public func collectionView(_ collectionView: UICollectionView, numberOfItemsInSection section: Int) -> Int {
        return generateStyleModels.count
    }
    
    public func collectionView(_ collectionView: UICollectionView, cellForItemAt indexPath: IndexPath) -> UICollectionViewCell {
        let cell = collectionView.dequeueReusableCell(withReuseIdentifier: APRingStyleCell.cellID, for: indexPath) as! APRingStyleCell
        if let model = generateStyleModels.safeAny(At: indexPath.item){
            cell.model = model
        }
        return cell
    }

    public func collectionView(_ collectionView: UICollectionView, didSelectItemAt indexPath: IndexPath) {
        generateStyleIndex = indexPath.item
        if let model = generateStyleModels.safeAny(At: generateStyleIndex) {
            generateStyleModel = model
        }
    }
    
}

class APRingStyleCell: ASBaseCollectionCell {
    static let cellID = "APRingStyleCell"
    var model: ASGenerateStyleModel = ASGenerateStyleModel()  {
        didSet {
            proImageView.isHidden = !model.vip
            textLab.text = model.imageText.localized
            imageView.image = UIImage(named: model.imageName)
        }
    }
    
    override var isSelected: Bool{
        didSet{
            textLab.textColor = .white.withAlphaComponent( isSelected ? 1.0 : 0.7)
            bgImageView.image = isSelected ? .ringTextSelectedBg : .ringTextUnSelectBg
            boardImageView.isHidden = !isSelected
        }
    }
    
    lazy var bgImageView: UIImageView = {
        let bgImageView = UIImageView.createImageV(image: .ringTextUnSelectBg)
        return bgImageView
    }()
    
    lazy var boardImageView: UIImageView = {
        let boardImageView = UIImageView.createImageV(image: .ringTextSelectedBoard)
        boardImageView.isHidden = true
        return boardImageView
    }()

    lazy var imageView: UIImageView = {
        let imageView = UIImageView()
        imageView.cornerRadius = 12
        return imageView
    }()
    
    lazy var proImageView:UIImageView = {
        let proImageView = UIImageView.createImageV(image: .PRO)
        return proImageView
    }()
    

    lazy var textLab: UILabel = {
        let textLab = UILabel.createLabel(font: .font(size:14,weight: .regular),textColor: .white.withAlphaComponent(0.6),textAlignment: .center)
        return textLab
    }()
    
    
    override func creatUI() {
        
        contentView.addSubview(bgImageView)
        bgImageView.snp.makeConstraints { make in
            make.edges.equalToSuperview()
        }
        
        contentView.addSubview(imageView)
        imageView.snp.makeConstraints { make in
            make.edges.equalToSuperview()
        }
        

        contentView.addSubview(textLab)
        textLab.snp.makeConstraints { make in
            make.leading.equalTo(0)
            make.trailing.equalTo(0)
            make.bottom.equalTo(0)
            make.height.equalTo(36)
        }
        
        contentView.addSubview(proImageView)
        proImageView.snp.makeConstraints { make in
            make.top.equalTo(6)
            make.trailing.equalTo(-8)
            make.width.equalTo(21)
            make.height.equalTo(10)
        }
 
        contentView.addSubview(boardImageView)
        boardImageView.snp.makeConstraints { make in
            make.edges.equalToSuperview()
        }
    
    }
}
