//
//  APRingTonesVM.swift
//  AIPlayRingtones
//
//  Created by 100Years on 2025/5/15.
//


let kRandomTextToRintone:[String] = [
    "Create a upbeat Pop ringtone with a catchy melody, bright synths, and a cheerful vibe. Use a BPM of 120-130 and keep it energetic and memorable.",
    "Generate a cinematic ringtone with orchestral strings, powerful brass, and a dramatic build-up. Use a BPM of 60-80 for a grand, inspiring feel.",
    "Produce a Lo-Fi ringtone with a relaxed piano melody, soft beats, and warm vinyl crackle. Use a BPM of 70-90 for a cozy, laid-back vibe.",
    "Create a EDM ringtone with a pulsating bassline, uplifting synths, and a quick build-up. Use a BPM of 128-132 for a high-energy, festival-ready sound.",
    "Generate a Jazz ringtone with a smooth saxophone melody, soft piano chords, and a gentle swing rhythm. Use a BPM of 90-100 for a classy, sophisticated tone.",
    "Produce a Funk ringtone with a groovy bassline, rhythmic guitar chops, and punchy brass hits. Use a BPM of 100-110 for a lively, danceable vibe.",
    "Create a Dubstep ringtone with heavy bass wobbles, aggressive synths, and a quick drop. Use a BPM of 140-150 for an intense, edgy sound.",
    "Generate a Folk ringtone with a fingerpicked acoustic guitar melody and light percussion. Use a BPM of 80-90 for a warm, earthy feel.",
    "Produce a Synthwave ringtone with retro arpeggiated synths, a driving beat, and an 80s-inspired vibe. Use a BPM of 100-110 for a nostalgic, futuristic tone.",
    "Create a Techno ringtone with a repetitive bassline, crisp hi-hats, and subtle synth textures. Use a BPM of 125-130 for a sleek, modern sound.",
]
import Combine
import ObjectMapper
class APRingTonesVM {
    private var cancellables = Set<AnyCancellable>()
    //输入框内容
    @Published var promptText:String = ""
    

    //选择生成时长类型组
    var generateSecIndex:Int = 0
    @Published var generateSecModel:ASRingGenerateSecModel?
    lazy var generateSecModels: [ASRingGenerateSecModel] = {
        var ptpStyleModels = [ASRingGenerateSecModel]()
        if let dataArray = Mapper<ASRingGenerateSecModel>().mapArray(JSONfile: "ring_duration.json"){
            ptpStyleModels = dataArray
            
            if let model = dataArray.safeAny(At: generateSecIndex) {
                generateSecModel = model //加上默认的选择
            }
        }
        
        return ptpStyleModels
    }()
    
    
    //选择生成类类型
    @Published var selectedGenerateModel:ASGenerateStyleModel?
    
    @Published private(set) var isCanGennerate: Bool = false
    init(){
        
       
        Publishers.CombineLatest3($promptText, $generateSecModel,$selectedGenerateModel)
              .map { promptText, generateSecModel,generateStyleModel in
                  // 当两者长度都 > 0 时返回 true
                  return !(promptText.replacingOccurrences(of: " ", with: "").isEmpty) && generateSecModel != nil && generateStyleModel != nil
              }
              .assign(to: \.isCanGennerate, on: self)
              .store(in: &cancellables)
    }

    
    //用户生成图片的TSGenerateStyleModel
    var generateStyleModel:ASGenerateStyleModel{
        let generateStyleModel = selectedGenerateModel!
        generateStyleModel.inputText = promptText
        generateStyleModel.duration = generateSecModel?.duration ?? 5
        return generateStyleModel
    }
}
