//
//  ASDurationColView.swift
//  AIPlayRingtones
//
//  Created by 100Years on 2025/5/15.
//

class ASGenerateDurationColView: ASBaseView {
    
    var selectedCompleted:((ASRingGenerateSecModel,Int)->Void)?
    
    var currentSelectedIndex:Int = 0 {
        didSet{
            if self.secModelArray.count <= 0 { return }
            collectionView.selectItem(at: IndexPath(item: currentSelectedIndex, section: 0), animated: true, scrollPosition: .centeredHorizontally)
        }
    }
    var secModelArray:[ASRingGenerateSecModel] = [] {
        didSet{
            collectionView.reloadData()
            let index = currentSelectedIndex
            kExecuteOnMainThread {
                self.currentSelectedIndex = index
            }
        }
    }

    lazy var layout: UICollectionViewFlowLayout = {
        let layout = UICollectionViewFlowLayout()
        layout.scrollDirection = .horizontal
        layout.itemSize = CGSize(width: 88, height: 36)
        layout.minimumInteritemSpacing = 0
        layout.minimumLineSpacing = 16
        return layout
    }()
    
    lazy var collectionView: UICollectionView = {
        let collectionView = UICollectionView(frame: .zero, collectionViewLayout: layout)
        collectionView.isScrollEnabled = false
        collectionView.delegate = self
        collectionView.dataSource = self
        collectionView.showsVerticalScrollIndicator = false
        collectionView.showsHorizontalScrollIndicator = false
        collectionView.backgroundColor = .clear
        collectionView.contentInset = UIEdgeInsets(top: 0, left: 16, bottom: 0, right: 16)
        collectionView.register(ASDurationColViewCell.self, forCellWithReuseIdentifier: ASDurationColViewCell.cellID)
        if #available(iOS 11.0, *) {
            collectionView.contentInsetAdjustmentBehavior = .never
        }
        return collectionView
    }()
    
    override func creatUI(){
        contentView.addSubview(collectionView)
        collectionView.snp.makeConstraints { make in
            make.edges.equalToSuperview()
        }
    }
}

extension ASGenerateDurationColView: UICollectionViewDataSource ,UICollectionViewDelegate,UICollectionViewDelegateFlowLayout {
    
    public func numberOfSections(in collectionView: UICollectionView) -> Int {
        return 1
    }
    
    public func collectionView(_ collectionView: UICollectionView, numberOfItemsInSection section: Int) -> Int {
        return secModelArray.count
    }
    
    public func collectionView(_ collectionView: UICollectionView, cellForItemAt indexPath: IndexPath) -> UICollectionViewCell {
        let cell = collectionView.dequeueReusableCell(withReuseIdentifier: ASDurationColViewCell.cellID, for: indexPath) as! ASDurationColViewCell
        if let model = secModelArray.safeAny(At: indexPath.item){
            cell.model = model
        }
        return cell
    }

    public func collectionView(_ collectionView: UICollectionView, didSelectItemAt indexPath: IndexPath) {
        currentSelectedIndex = indexPath.row
        if let model = secModelArray.safeAny(At: currentSelectedIndex) {
            selectedCompleted?(model,currentSelectedIndex)
        }
    }

}


class ASDurationColViewCell: ASBaseCollectionCell {
    static let cellID = "ASDurationColViewCell"
    var model: ASRingGenerateSecModel = ASRingGenerateSecModel()  {
        didSet {
            textLab.text = "\(model.duration) Sec"
            proImageView.isHidden = !model.vip
        }
    }
    override var isSelected: Bool{
        didSet{
            bgImageView.image = isSelected ? .secSelectedBg : .secUnSelectedBg
            textLab.font = .font(size:14,weight: isSelected ? .medium : .regular)
            textLab.textColor = .white.withAlphaComponent( isSelected ? 1.0 : 0.6)
        }
    }
    
    lazy var proImageView:UIImageView = {
        let proImageView = UIImageView.createImageV(image: .PRO)
        return proImageView
    }()
    
    lazy var bgImageView: UIImageView = {
        let bgImageView = UIImageView.createImageV(image: .secUnSelectedBg)
        return bgImageView
    }()
    
    lazy var textLab: UILabel = {
        let textLab = UILabel.createLabel(font: .font(size:14,weight: .regular),textColor: .white.withAlphaComponent(0.6),textAlignment: .center)
        return textLab
    }()
    
    
    override func creatUI() {
        
        contentView.addSubview(bgImageView)
        bgImageView.snp.makeConstraints { make in
            make.edges.equalToSuperview()
        }
        
        contentView.addSubview(textLab)
        textLab.snp.makeConstraints { make in
            make.centerY.equalToSuperview()
            make.leading.equalTo(8)
            make.trailing.equalTo(-8)
            make.height.equalTo(20)
        }
        
        contentView.addSubview(proImageView)
        proImageView.snp.makeConstraints { make in
            make.top.equalTo(4)
            make.trailing.equalTo(-4)
            make.width.equalTo(21)
            make.height.equalTo(10)
        }
    }
}
