//
//  ASPromptTextView.swift
//  AIPlayRingtones
//
//  Created by 100Years on 2025/5/15.
//


class ASPromptTextView : ASBaseView{
    
    var randomTextArray:[String]
    var textChangedBlock:((String)->Void)
    
    init(randomTextArray: [String], textChangedBlock: @escaping (String) -> Void) {
        self.randomTextArray = randomTextArray
        self.textChangedBlock = textChangedBlock
        super.init(frame: .zero)
    }
    
    @MainActor required init?(coder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
    

    lazy var randomTextPicker: ASRandomTextTool = {
        let textPicker = ASRandomTextTool(texts: randomTextArray)
        return textPicker
    }()
    
    lazy var boardImageView: UIImageView = {
        let boardImageView = UIImageView.createImageV(imageName: "textView_board",contentMode: .scaleToFill)
        boardImageView.isHidden = true
        return boardImageView
    }()
    
    lazy var textBgView: UIView = {
        let textBgView = UIView()
        textBgView.cornerRadius = 16.0
//        let bgImageView = UIImageView.createImageV(imageName: "textView_bg",contentMode: .scaleToFill)
//        textBgView.addSubview(bgImageView)
//        bgImageView.snp.makeConstraints { make in
//            make.edges.equalToSuperview()
//        }
        kAddCommonGradientBg(view: textBgView)
        textBgView.addSubview(boardImageView)
        boardImageView.snp.makeConstraints { make in
            make.edges.equalToSuperview()
        }
        
        return textBgView
    }()
    
    lazy var customTextView: ASInputView = {
        let customTextView = ASInputView(
            hint: "Type your idea here.",
            textFont: .font(size: 14),
            contentColor: .white,
            background: .clear
        )
        customTextView.delegate = self
        customTextView.returnKeyType = .done
        customTextView.onTextUpdate = { [weak self]  in
            guard let self = self else { return }
            textViewTextChanged(textView: customTextView)
        }
        return customTextView
    }()
    
 
    lazy var inspirationBtn: UIButton = {
        let inspirationBtn = UIButton.createBtn(
            title: "💡Get Inspired?".localized,
            font: .font(size: 12),
            titleColor: "#F3F0EA".uiColor
        )
        { [weak self]  in
            guard let self = self else { return }
            customTextView.text = randomTextPicker.getRandomText()
            textViewTextChanged(textView: customTextView)
        }
//        inspirationBtn.contentEdgeInsets = UIEdgeInsets(top: 4, left: 40, bottom: 4, right: 50)
//        inspirationBtn.imageEdgeInsets = UIEdgeInsets(top: 0, left: -10, bottom: 0, right: 0)
        return inspirationBtn
    }()
    
    
    lazy var clearBtn: ASTouchBtn = {
        let clearBtn = ASTouchBtn()
        clearBtn.setUpBtn(image: UIImage(named: "clear_text"))
        { [weak self]  in
            guard let self = self else { return }
            customTextView.text = ""
            textViewTextChanged(textView: customTextView)
        }
        clearBtn.setImage(UIImage(named: "clear_gary_text"), for: .disabled)
        clearBtn.isEnabled = false
        return clearBtn
    }()
    
    let maxNum:Int = 1000
    var currenMun:Int = 0 {
        didSet{
            currenMunLab.text = "\(currenMun)"
        }
    }
    lazy var currenMunLab: UILabel = {
        let numLab = UILabel.createLabel(text: "\(currenMun)",font: .font(name: .PingFangSC,size: 12),textColor: .white.withAlphaComponent(0.8))
        return numLab
    }()
    
    lazy var numView: UIView = {
        let numView = UIView()
        
        numView.addSubview(currenMunLab)
        
        let numLab = UILabel.createLabel(text: "/\(maxNum)",font: .font(name: .PingFangSC,size: 12),textColor: .white.withAlphaComponent(0.4))
        numView.addSubview(numLab)
        
        let lineView = UIView()
        lineView.cornerRadius = 0.5
        lineView.backgroundColor = .white
        numView.addSubview(lineView)
        
        currenMunLab.snp.makeConstraints { make in
            make.centerY.equalToSuperview()
            make.leading.equalTo(0)
        }
        
        numLab.snp.makeConstraints { make in
            make.centerY.equalToSuperview()
            make.leading.equalTo(currenMunLab.snp.trailing)
        }
        
        lineView.snp.makeConstraints { make in
            make.top.trailing.bottom.equalToSuperview()
            make.leading.equalTo(numLab.snp.trailing).offset(8)
            make.width.equalTo(1)
            make.height.equalTo(12)
        }
        
        return numView
    }()
    
    override func creatUI() {
        contentView.addSubview(textBgView)
        textBgView.snp.makeConstraints { make in
            make.top.equalTo(0)
            make.leading.equalTo(16)
            make.trailing.equalTo(-16)
            make.height.equalTo(114.0*kDesignScale)
            make.bottom.equalTo(0)
        }
   
        textBgView.addSubview(customTextView)
        customTextView.snp.makeConstraints { make in
            make.top.equalTo(16)
            make.leading.equalTo(12)
            make.trailing.equalTo(-12)
            make.bottom.equalTo(-42)
        }
        
        textBgView.addSubview(inspirationBtn)
        inspirationBtn.snp.makeConstraints { make in
            make.height.equalTo(12)
            make.bottom.equalTo(-12)
            make.leading.equalTo(12)
        }
        
        textBgView.addSubview(clearBtn)
        clearBtn.snp.makeConstraints { make in
            make.width.height.equalTo(16)
            make.bottom.equalTo(-10)
            make.trailing.equalTo(-12)
        }
        
        textBgView.addSubview(numView)
        numView.snp.makeConstraints { make in
            make.centerY.equalTo(clearBtn)
            make.trailing.equalTo(clearBtn.snp.leading).offset(-8)
        }
    
    }

    func getVipText()->String{
        return "Generate"
    }
    
    func textViewTextChanged(textView: UITextView) {
        currenMun = textView.text.count
        clearBtn.isEnabled = currenMun > 0
        boardImageView.isHidden = !clearBtn.isEnabled
        textChangedBlock(textView.text)
    }
}

extension ASPromptTextView: UITextViewDelegate{
     func textView(_ textView: UITextView, shouldChangeTextIn range: NSRange, replacementText text: String) -> Bool {
         if text == "\n" {
             textView.resignFirstResponder()
             return false
         }
         return true
     }
    
}
