//
//  APSettingVC+Col.swift
//  AIPlayRingtones
//
//  Created by mini on 2025/5/20.
//

extension APSettingVC {
 
    func creatCollectionView() -> UICollectionView{
        let layout = UICollectionViewFlowLayout()
        layout.scrollDirection = .vertical
        layout.itemSize = CGSize(width: k_ScreenWidth-32, height: 64)
        layout.minimumInteritemSpacing = 16.0
        layout.minimumLineSpacing = 16.0
        
        layout.sectionInset = UIEdgeInsets(top: 16, left: 16, bottom: 16, right: 16)
        
        let collectionView = UICollectionView(frame: .zero, collectionViewLayout: layout)
        collectionView.delegate = self
        collectionView.dataSource = self
        collectionView.showsVerticalScrollIndicator = false
        collectionView.showsHorizontalScrollIndicator = false
        collectionView.backgroundColor = .clear
        collectionView.register(APSettingCell.self, forCellWithReuseIdentifier: APSettingCell.cellID)
        if #available(iOS 11.0, *) {
            collectionView.contentInsetAdjustmentBehavior = .never
        }
        return collectionView
    }
    
    func createItemModel(leftTitle: String,
                         rightIsHave: Bool,
                        tapBlock: @escaping ((ASItemModel, Int, Any?) -> Void)) -> ASItemModel {
        let model = ASItemModel()
        model.leftTitle = leftTitle
        model.rightIsHave = rightIsHave
        model.tapBlock = tapBlock
        return model
    }
    
}

extension APSettingVC: UICollectionViewDataSource ,UICollectionViewDelegate {
    
    public func numberOfSections(in collectionView: UICollectionView) -> Int {
        return dataArray.count
    }
    
    public func collectionView(_ collectionView: UICollectionView, numberOfItemsInSection section: Int) -> Int {
        if let sectionModel = dataArray.safeAny(At: section){
            return sectionModel.itemsArray.count
        }
        return 0
    }
    
    public func collectionView(_ collectionView: UICollectionView, cellForItemAt indexPath: IndexPath) -> UICollectionViewCell {
        
        let cell = collectionView.dequeueReusableCell(withReuseIdentifier: APSettingCell.cellID, for: indexPath)
        if let cell = cell as? APSettingCell {
            if let sectionModel = dataArray.safeAny(At: indexPath.section),let itemModel = sectionModel.itemsArray.safeAny(At: indexPath.row){
                cell.itemModel = itemModel
            }
        }
        
        return cell
    }

    public func collectionView(_ collectionView: UICollectionView, didSelectItemAt indexPath: IndexPath) {
        if let sectionModel = dataArray.safeAny(At: indexPath.section),let itemModel = sectionModel.itemsArray.safeAny(At: indexPath.row){
            itemModel.tapBlock?(itemModel,indexPath.row,nil)
        }
    }
}


class APSettingCell: ASBaseCollectionCell {
    
    static let cellID = "APSettingCell"
    var itemModel:ASItemModel = ASItemModel(){
        didSet{
            titleLab.text = itemModel.leftTitle
            if itemModel.rightIsHave {
                infoLab.isHidden = true
                arrowImageView.isHidden = false
            }else{
                infoLab.isHidden = false
                arrowImageView.isHidden = true
            }
        }
    }
    
    lazy var arrowImageView: UIImageView = {
        let arrowImageView = UIImageView.createImageV(image: .rightArrowWhite)
        return arrowImageView
    }()

    
    lazy var titleLab: UILabel = {
        let titleLab = UILabel.createLabel(font: .font(size: 16,weight: .regular),textColor: .white,textAlignment: .left,numberOfLines: 0)
        return titleLab
    }()
    
    lazy var infoLab: UILabel = {
        let infoLab = UILabel.createLabel(text: appVersion(),font: .font(size: 16,weight: .regular),textColor: .white.withAlphaComponent(0.8),numberOfLines: 0)
        return infoLab
    }()
    
    override func creatUI() {
        bgContentView.clipsToBounds = true
        bgContentView.cornerRadius = 16.0
//        bgContentView.layer.borderWidth = 1
//        bgContentView.layer.borderColor = UIColor.white.withAlphaComponent(0.2).cgColor
        bgContentView.backgroundColor = "#202030".uiColor
        
        
        bgContentView.addSubview(titleLab)
        titleLab.snp.makeConstraints { make in
            make.centerY.equalToSuperview()
            make.leading.equalTo(16)
            make.trailing.equalTo(-40)
        }
        
        bgContentView.addSubview(infoLab)
        infoLab.snp.makeConstraints { make in
            make.centerY.equalToSuperview()
            make.trailing.equalTo(-16)
        }

        bgContentView.addSubview(arrowImageView)
        arrowImageView.snp.makeConstraints { make in
            make.trailing.equalTo(-16)
            make.centerY.equalToSuperview()
            make.width.height.equalTo(24.0)
        }
    }
    
}

