//
//  APSettingVC.swift
//  AIPlayRingtones
//
//  Created by 100Years on 2025/5/15.
//

import StoreKit

class APSettingVC: ASBaseViewController {
    
    lazy var navBarView: ASBaseNavContentBarView = {
        let navBarView = ASBaseNavContentBarView()

        let titleImageView = UIImageView.createImageV(imageName: "nav_title_setting", contentMode: .scaleToFill)
        navBarView.barView.addSubview(titleImageView)
        titleImageView.snp.makeConstraints { make in
            make.center.equalToSuperview()
        }
        return navBarView
    }()
    
    lazy var dynamicContainer: ASDynamicContainer = {
        let dynamicContainer = ASDynamicContainer(orientation: .vertical)
        return dynamicContainer
    }()
    
    private var collectionViewObserver: ASColObserver!
    lazy var collectionView = creatCollectionView()
    
    
    let appid = "6746234900"
    lazy var dataArray: [ASSectionModel] = {
        var dataArray = [ASSectionModel]()
        let sectionModel = ASSectionModel()
        dataArray.append(sectionModel)

        //教程
        sectionModel.itemsArray.append(
            createItemModel(
                leftTitle: "How to use?".localized,
                rightIsHave: true,
                tapBlock: { [weak self] model, _, _ in
                   guard let self = self else { return }
                    kPushVC(target: self, modelVC: ASTutorialsVC())
        }))
        
        //评价
        sectionModel.itemsArray.append(
            createItemModel(
                leftTitle: "Rate us".localized,
                rightIsHave: true,
                tapBlock: { [weak self] model, _, _ in
                   guard let self = self else { return }
                    
                    let countKey = "ProcessCompletedCountKey"
                    var count = UserDefaults.standard.integer(forKey: countKey)
                    count += 1
                    UserDefaults.standard.set(count, forKey: countKey)
                    /// 如果当前版本 弹过一次 那么下一次就走url
                    /// 如果当前版本 没弹过，那就弹
                    if count >= 4 {
                        let openStr = "itms-apps://itunes.apple.com/app/\(appid)?action=write-review"
                        if let url = URL(string: openStr), UIApplication.shared.canOpenURL(url) {
                            if #available(iOS 10.0, *) {
                                UIApplication.shared.open(url, options: [:], completionHandler: nil)
                            } else {
                                UIApplication.shared.canOpenURL(url)
                            }
                        } else {
                            print("链接出错")
                        }
                    } else {
                        SKStoreReviewController.requestReview()
                    }
      
        }))
        
        //分享
        sectionModel.itemsArray.append(
            createItemModel(
                leftTitle: "Share us".localized,
                rightIsHave: true,
                tapBlock: { [weak self] model, _, _ in
                   guard let self = self else { return }
                    
                    let httpAppStoreLink = "https://apps.apple.com/app/id\(appid)"
                    let final = ShareActivityItemProvider(placeholderItem: UIImage(named: "App-Icon")!.compressSize(to: CGSize(width: 100, height: 100)))
                    let url = URL(string: httpAppStoreLink)!
                    let vc = UIActivityViewController(activityItems: [url, final], applicationActivities: nil)
                    vc.completionWithItemsHandler = { activity, _, _, _ in
                        if let type = activity, type == .copyToPasteboard {
                            UIPasteboard.general.string = httpAppStoreLink
                        }
                    }

                    present(vc, animated: true)
      
        }))
        
        //隐私协议
        sectionModel.itemsArray.append(
            createItemModel(
                leftTitle: "Privacy Policy".localized,
                rightIsHave: true,
                tapBlock: { [weak self] model, _, _ in
                guard let self = self else { return }
                 let vc = ASWebViewController(urlString: "https://doc-hosting.flycricket.io/ai-ringtones-privacy-policy/ad14f989-96c0-4f31-a711-8db8b288d0e2/privacy", titleString: "Privacy Policy".localized)!
                 vc.hidesBottomBarWhenPushed = true
                 navigationController?.pushViewController(vc, animated: true)
        }))
        
        //服务条框
        sectionModel.itemsArray.append(
            createItemModel(
                leftTitle: "Terms of Service".localized,
                rightIsHave: true,
                tapBlock: { [weak self] model, _, _ in
                   guard let self = self else { return }
                    let vc = ASWebViewController(urlString: "https://doc-hosting.flycricket.io/ai-ringtones-terms-of-use/4f89d174-1559-453f-b12c-47a85e21cc01/terms", titleString: "Terms of Service".localized)!
                    vc.hidesBottomBarWhenPushed = true
                    navigationController?.pushViewController(vc, animated: true)
        }))
        
        //关于我们
        sectionModel.itemsArray.append(
            createItemModel(
                leftTitle: "About us".localized,
                rightIsHave: false,
                tapBlock: { [weak self] model, _, _ in
                   guard let self = self else { return }
                    kSuccessManager.displayNotification(in: self.view,message: "Successfully generated".localized)
        }))
        
        return dataArray

    }()
    
    override func createView() {
        
        navBarContentView.addSubview(navBarView)
        navBarView.snp.makeConstraints { make in
            make.edges.equalToSuperview()
        }
        
        contentView.addSubview(dynamicContainer)
        dynamicContainer.snp.makeConstraints { make in
            make.edges.equalToSuperview()
        }
        
        dynamicContainer.embedView(collectionView,dimension: 0)
    }
    
    override func dealThings() {

        // 监听collectionView 的 contentSize
        collectionViewObserver = ASColObserver(collectionView: collectionView)
        collectionViewObserver.onContentSizeChange = { [weak self] size in
            guard let self = self else { return }
            print("collectionViewObserver 内容大小变化: \(size)")
            self.collectionView.snp.updateConstraints { make in
                make.height.equalTo(size.height)
            }
        }
        collectionView.reloadData()
        
    }
    
}


class ShareActivityItemProvider: UIActivityItemProvider {
    // 解决AirDrop分享app失败的问题
    override func activityViewController(_ activityViewController: UIActivityViewController, itemForActivityType activityType: UIActivity.ActivityType?) -> Any? {
        if activityType == .airDrop {
            return nil
        }
        return placeholderItem
    }
}
