//
//  ASRingGeneratorVC.swift
//  AIPlayRingtones
//
//  Created by 100Years on 2025/5/18.
//
import Combine
class ASRingGeneratorVC: ASRingGeneratorBaseVC {
    public var cancellable: [AnyCancellable] = []
    
    var infoModel:ASActionInfoModel?
    var complete:((ASActionInfoModel)->Void)?
    var reloadViewBlock:(()->Void)?
    var progressState = ASProgressState.none


    var generateStyleModel:ASGenerateStyleModel
    init(generateStyleModel:ASGenerateStyleModel,infoModel:ASActionInfoModel? = nil,complete: ((ASActionInfoModel)->Void)? = nil) {
        self.generateStyleModel = generateStyleModel
        self.infoModel = infoModel
        self.complete = complete
        super.init()
    }
    
    @MainActor required init?(coder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
    
    lazy var generateInView : ASGeneratorView = createGeneratorView()
    lazy var mediaPlayer = ASMediaPlayerController.main
    var uuidString:String = UUID().uuidString

    lazy var ringView: ASRingToneCellView = {
        let ringToneView = ASRingToneCellView()
        ringToneView.isSelectedChangeColor = false
        ringToneView.nameLab.snp.updateConstraints { make in
            make.trailing.equalTo(-16)
        }
        ringToneView.clickPlayHandel = { [weak self] play  in
            guard let self = self else { return }
            clickPlay()
        }
        return ringToneView
    }()
    
    var operation:ASGenerateBaseOperation?
    override func createView() {
        super.createView()
        
        view.addSubview(generateInView)
        generateInView.snp.makeConstraints { make in
            make.top.bottom.leading.trailing.equalTo(0)
        }

        bottomView.addSubview(ringView)
        ringView.snp.makeConstraints { make in
            make.top.equalTo(82)
            make.centerX.equalToSuperview()
            make.width.equalTo(k_ScreenWidth - 32)
            make.height.equalTo(72)
        }
    }
    
    @objc func clickBackstageBtn() {
        self.operation?.backstageGeneration()
        NotificationCenter.default.post(name: .kRingDataChanged, object: nil)
        self.dismiss(animated: true, completion: nil)
    }
    
    override func closePage() {
        if progressState.isResult {
            self.dismiss(animated: true, completion: nil)
        }else{
            ASCustomAlert.show(in: self, config: ASCustomAlert.AlertConfig(
                message: "As you leave, your generation will be interrupted and no result.".localized,
                messageColor: .white,
                messageFont: .systemFont(ofSize: 16),
                
                cancelTitle: "Leave".localized,
                cancelColor: .white.withAlphaComponent(0.6),
                
                confirmTitle: "Wait".localized,
                confirmColor: .themeColor,
                
                cancelAction: { [weak self]  in
                    guard let self = self else { return }
                    print("用户点击了Leave")
                    operation?.cancel()
                    if let model = infoModel{
                        dbHistory.deleteListModel(id: model.id)
                        NotificationCenter.default.post(name: .kRingDataChanged, object: nil)
                    }
                    self.dismiss(animated: true, completion: nil)
                },
                confirmAction: {
                    print("用户点击了Stay")
                }
            ))
        }
    }
    

    @objc override func clickSubmitBtn(){
        
        guard let model = infoModel else {
            kShowToastDataMissing()
            return
        }
        mediaPlayer.stopPlayback()

        ASMediaDownloader.fetchMediaResource(remoteURL: model.response.musicUrl){ url,downLoad in
            if let path = url {
                _ = kshareBand(needVip: model.response.vip, vc: self, fileURL: path, fileName: model.response.title)
            }
        }
    }
    
    @objc func clickPlay(){
        
        guard let infoModel = infoModel else { return }
        
        if  mediaPlayer.isCurrentMedia(url:infoModel.response.musicUrl) {
            if mediaPlayer.isBuffering{
                return
            }
            if mediaPlayer.isActive{
                mediaPlayer.stopPlayback()
            }else{
                mediaPlayer.startPlayback(of:infoModel.response.musicUrl)
            }
        }else {
            mediaPlayer.stopPlayback()//让上一个对象停止播放
            ringView.monitorPlayStateDefaultHandle()//更换监听
            mediaPlayer.startPlayback(of:infoModel.response.musicUrl)
        }

    }
    
    override func viewWillDisappear(_ animated: Bool) {
        super.viewWillDisappear(animated)
        mediaPlayer.stopPlayback()
    }
    
    override func dealThings() {
        self.uuidString = UUID().uuidString
        creatRintone()
    }

 
    func creatRintone() {
        if let model = self.infoModel{
            if model.actionStatus == .failed{
                generatorCreatRing(oldModel: model)
            }else if model.response.musicUrl.count > 0 {
                upDateView(state: .success(model), model: model)
                return
            }
        }else{
            generatorNew()
        }
//        if kPurchaseToolShared.kJudgeVipFreeType(vipFreeNumType: .ringtones, vc: self){ return }//判断 vip
    }

    deinit {
        cancellable.removeAll()
    }
}

extension ASRingGeneratorVC{
    
    var dbHistory:ASDBHistory{
        ASRMShared.ringDBHistory
    }

    func generatorNew(){
        if generateStyleModel.generateType == .textToRing {
            generatorNewTTR()
        }else{
            generatorNewRTR()
        }
    }
    
    func generatorCreatRing(oldModel:ASActionInfoModel) {
        if generateStyleModel.generateType == .textToRing {
            generatorTTRCreatRing(oldModel: oldModel)
        }else{
            generatorRTRCreatRing(oldModel: oldModel)
        }
    }
}

extension ASRingGeneratorVC{
    
    func generatorNewTTR() {
        let operation:ASGenerateTextToRingOperation = ASGenerateTextToRingOperationQueue.shared.creatOperation(uuid: self.uuidString)
        self.operation = operation
        let oldModel = operation.createActionInfoModel(generateStyleModel: generateStyleModel)
        generatorTTRCreatRing(oldModel: oldModel)
    }
    
    func generatorTTRCreatRing(oldModel:ASActionInfoModel) {
        let operation:ASGenerateTextToRingOperation = ASGenerateTextToRingOperationQueue.shared.creatOperation(uuid: self.uuidString)
        self.operation = operation
        operation.$stateDatauPblished.receive(on: DispatchQueue.main).sink {[weak self]  (state,model) in
            guard let self = self else { return }
            self.upDateView(state: state, model: model)
        }.store(in: &cancellable)
        
        operation.creatRing(oldModel: oldModel){ [weak self]  success in
            guard let self = self else { return }
            if success {
                generateInView.setBackgroundGenerateBtnHidden(false)
                xBtn.isHidden = false
            }
        }
    }
}

extension ASRingGeneratorVC{
    func generatorNewRTR() {
        let operation:ASGenerateRingToRingOperation = ASGenerateRingToRingOperationQueue.shared.creatOperation(uuid: self.uuidString)
        self.operation = operation
        operation.$stateDatauPblished.receive(on: DispatchQueue.main).sink {[weak self]  (state,model) in
            guard let self = self else { return }
            self.upDateView(state: state, model: model)
        }.store(in: &cancellable)
        
        operation.uploadFile(generateStyleModel: generateStyleModel)  { [weak self] actionInfoModel in
            guard let self = self else { return }
            guard let oldModel = actionInfoModel else {return}
            generatorRTRCreatRing(oldModel: oldModel)
        }

    }
    
    func generatorRTRCreatRing(oldModel:ASActionInfoModel) {
        let operation:ASGenerateRingToRingOperation = ASGenerateRingToRingOperationQueue.shared.creatOperation(uuid: self.uuidString)
        self.operation = operation
        operation.$stateDatauPblished.receive(on: DispatchQueue.main).sink {[weak self]  (state,model) in
            guard let self = self else { return }
            self.upDateView(state: state, model: model)
        }.store(in: &cancellable)
        
        operation.creatRing(oldModel: oldModel){ [weak self]  success in
            guard let self = self else { return }
            if success {
                generateInView.setBackgroundGenerateBtnHidden(false)
                xBtn.isHidden = false
            }
        }
    }
}


extension ASRingGeneratorVC{

    func updateInfoModel(model:ASActionInfoModel?){
        infoModel = model
        if let model = model {
            complete?(model)
            setRingViewData(model: model)
        }
    }

    func setRingViewData(model:ASActionInfoModel) {
        ringView.timeLab.text = Float(model.request.duration).timeMMSS
        ringView.nameLab.text = model.response.title
        ringView.setCoverImageView(urlString: model.response.coverUrl)
    }
}

