//
//  ASColObserver.m
//  AIPlayRingtones
//
//  Created by mini on 2025/5/28.
//

#import "ASColObserver.h"
#import <objc/runtime.h>

@interface ASColObserver ()

@property (nonatomic, weak) UICollectionView *collectionView;
@property (nonatomic, strong) NSMutableArray *observers;

@end

@implementation ASColObserver


- (instancetype)initWithCollectionView:(UICollectionView *)collectionView {
    self = [super init];
    if (self) {
        _collectionView = collectionView;
        _observers = [NSMutableArray array];
        [self setupObservers];
    }
    return self;
}

- (void)setupObservers {
    if (!self.collectionView) return;
    
    // ContentSize Observer
    [self.collectionView addObserver:self
                         forKeyPath:@"contentSize"
                            options:NSKeyValueObservingOptionNew | NSKeyValueObservingOptionOld
                            context:nil];
    
    // ContentInset Observer
    [self.collectionView addObserver:self
                         forKeyPath:@"contentInset"
                            options:NSKeyValueObservingOptionNew | NSKeyValueObservingOptionOld
                            context:nil];
    
    // ContentOffset Observer
    [self.collectionView addObserver:self
                         forKeyPath:@"contentOffset"
                            options:NSKeyValueObservingOptionNew
                            context:nil];
}

- (void)observeValueForKeyPath:(NSString *)keyPath
                      ofObject:(id)object
                        change:(NSDictionary<NSKeyValueChangeKey,id> *)change
                       context:(void *)context {
    if (object == self.collectionView) {
        if ([keyPath isEqualToString:@"contentSize"]) {
            CGSize newSize = [change[NSKeyValueChangeNewKey] CGSizeValue];
            CGSize oldSize = [change[NSKeyValueChangeOldKey] CGSizeValue];
            if (!CGSizeEqualToSize(newSize, oldSize) && self.onContentSizeChange) {
                self.onContentSizeChange(newSize);
            }
        }
        else if ([keyPath isEqualToString:@"contentInset"]) {
            UIEdgeInsets newInset = [change[NSKeyValueChangeNewKey] UIEdgeInsetsValue];
            UIEdgeInsets oldInset = [change[NSKeyValueChangeOldKey] UIEdgeInsetsValue];
            if (!UIEdgeInsetsEqualToEdgeInsets(newInset, oldInset) && self.onContentInsetChange) {
                self.onContentInsetChange(newInset);
            }
        }
        else if ([keyPath isEqualToString:@"contentOffset"]) {
            CGPoint newOffset = [change[NSKeyValueChangeNewKey] CGPointValue];
            if (self.onContentOffsetChange) {
                self.onContentOffsetChange(newOffset);
            }
        }
    }
}

- (void)dealloc {
    [self.observers enumerateObjectsUsingBlock:^(id  _Nonnull obj, NSUInteger idx, BOOL * _Nonnull stop) {
        [self.collectionView removeObserver:self forKeyPath:obj];
    }];
}

@end
