//
//  ASMediaDownloader.swift
//  AIPlayRingtones
//
//  Created by mini on 2025/5/28.
//

import AVFoundation
import Alamofire

class ASMediaDownloader {
    
    // MARK: - 铃声下载
    
    static func fetchMediaResource(
        remoteURL: String,
        progressUpdate: ((Double) -> Void)? = nil,
        completion: @escaping (URL?, Bool) -> Void
    ) {
        if let cachedPath = ASResourceCacheHandler.locateCachedResource(
            originalURL: remoteURL,
            preferredExtension: nil,
            alternativeExtension: "mp3"
        ) {
            completion(cachedPath, false)
        } else {
            _ = initiateMediaDownload(
                sourceURL: remoteURL,
                fallbackExtension: "mp3",
                progressTracker: progressUpdate
            ) { destinationURL, error in
                completion(destinationURL, true)
            }
        }
    }
    
    // MARK: - 下载核心方法
    
    static func initiateMediaDownload(
        sourceURL: String,
        fileExtension: String? = nil,
        fallbackExtension: String? = nil,
        progressTracker: ((Double) -> Void)? = nil,
        onCompletion: @escaping (URL?, Error?) -> Void
    ) -> DownloadRequest? {
        
        // 检查缓存是否存在
        if let existingFile = ASResourceCacheHandler.locateCachedResource(
            originalURL: sourceURL,
            preferredExtension: fileExtension,
            alternativeExtension: fallbackExtension
        ) {
            onCompletion(existingFile, nil)
            return nil
        }
        
        // 准备存储路径
        guard let saveLocation = ASResourceCacheHandler.prepareCacheLocation(
            for: sourceURL,
            withExtension: fileExtension,
            defaultExtension: fallbackExtension,
            in: ASResourceCacheHandler.ringStoragePath,
            completion: { _, error in
                onCompletion(nil, error)
            }
        ) else {
            return nil
        }
        
        // 启动下载任务
        return ASNetworkShared.retrieveFile(from: sourceURL,saveTo: saveLocation, progressTracker:progressTracker,completion: onCompletion)
    }
}

// MARK: - 网络服务扩展
extension ASMediaDownloader {
    
    private enum Constants {
        static let audioExtension = "mp3"
    }
    
    static func downloadAudioResource(
        url: String,
        onProgress: ((Double) -> Void)? = nil,
        completion: @escaping (URL?, Bool) -> Void
    ) {
        fetchMediaResource(
            remoteURL: url,
            progressUpdate: onProgress,
            completion: completion
        )
    }
}
