//
//  ASWindTool.swift
//  AIPlayRingtones
//
//  Created by mini on 2025/5/28.
//


class ASWindTool {
    
    /// 获取当前的 keyWindow
    public static func getKeyWindow() -> UIWindow? {
        return UIApplication.shared.connectedScenes
                .compactMap { $0 as? UIWindowScene }
                .flatMap { $0.windows }
                .first { $0.isKeyWindow }
    }
    
    /// 获取当前窗口，兼容 iOS 13 及以上版本
    public static func getCurrentWindow() -> UIWindow? {
        guard let scene = UIApplication.shared.connectedScenes.first(where: { $0.activationState == .foregroundActive }) as? UIWindowScene else {
            return nil
        }
        return scene.windows.first { $0.isKeyWindow }
    }
    
    /// 获取当前根视图控制器
    public static func getRootViewController() -> UIViewController? {
        guard let window = getCurrentWindow() else {
            return nil
        }
        return window.rootViewController
    }
    
    /// 获取当前的视图控制器
    public static func getCurrentViewController() -> UIViewController? {
        var currentViewController = getRootViewController()
        while let presentedViewController = currentViewController?.presentedViewController {
            currentViewController = presentedViewController
        }
        return currentViewController
    }
    
    public static func topViewController(base: UIViewController? = UIApplication.shared.keyWindow?.rootViewController) -> UIViewController? {
        if let nav = base as? UINavigationController {
            return topViewController(base: nav.visibleViewController)
        } else if let tab = base as? UITabBarController {
            return topViewController(base: tab.selectedViewController)
        } else if let presented = base?.presentedViewController {
            return topViewController(base: presented)
        }
        return base
    }
}
