//
//  ASNormalNavigationBarView.m
//  AIPlayRingtones
//
//  Created by mini on 2025/5/27.
//

#import "ASNormalNavigationBarView.h"
#import "Masonry.h"
#import "NSString+AS.h"
#import "AIPlayRingtones-Swift.h"
@implementation ASNormalNavigationBarView


- (instancetype)initWithFrame:(CGRect)frame {
    self = [super initWithFrame:frame];
    if (self) {
        [self createData];
        [self createView];
    }
    return self;
}

- (void)createData {
    self.navBgColor = [UIColor clearColor];
    self.navTitleColor = [UIColor whiteColor];
    self.navTitleFont = [UIFont systemFontOfSize:18 weight:UIFontWeightMedium];
    self.navLeftFont = [UIFont systemFontOfSize:16 weight:UIFontWeightMedium];
    self.navRightFont = [UIFont systemFontOfSize:16 weight:UIFontWeightMedium];
}

- (void)createView {
    CGFloat margins = 16;
    
    self.backgroundColor = self.navBgColor;
    
    _leftNavBtn = [self createNavButton];
    _leftNavBtn.contentHorizontalAlignment = UIControlContentHorizontalAlignmentLeft;
    _leftNavBtn.contentEdgeInsets = UIEdgeInsetsZero;
    
    _titleNavBtn = [self createNavButton];
    _titleNavBtn.titleLabel.font = self.navTitleFont;
    
    _rightNavBtn = [self createNavButton];
    _rightNavBtn.titleLabel.font = self.navRightFont;
    _rightNavBtn.titleLabel.adjustsFontSizeToFitWidth = YES;
    [_rightNavBtn setTitleColor:@"B48EFF".uiColor forState:UIControlStateNormal];
    _rightNavBtn.contentHorizontalAlignment = UIControlContentHorizontalAlignmentRight;
    _rightNavBtn.contentEdgeInsets = UIEdgeInsetsZero;
    
    [self.barView addSubview:_leftNavBtn];
    [self.barView addSubview:_titleNavBtn];
    [self.barView addSubview:_rightNavBtn];
    
    [_leftNavBtn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.leading.equalTo(@(margins));
        make.centerY.equalTo(self.barView);
        make.width.height.equalTo(@44);
    }];
    
    [_rightNavBtn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.trailing.equalTo(@(-margins));
        make.centerY.equalTo(self.barView);
        make.width.height.equalTo(@44);
    }];
    
    [_titleNavBtn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.leading.equalTo(_leftNavBtn.mas_trailing).offset(margins);
        make.trailing.equalTo(_rightNavBtn.mas_leading).offset(-margins);
        make.centerY.equalTo(self.barView);
        make.height.equalTo(@44);
    }];
}

- (UIButton *)setTitleName:(NSString *)name {
    [self.titleNavBtn setTitle:name forState:UIControlStateNormal];
    return self.titleNavBtn;
}

- (UIButton *)setTitleNavigationItemWithName:(NSString *)name imageName:(NSString *)imageName target:(id)target action:(SEL)action {
    return [self setNavButton:self.titleNavBtn name:name imageName:imageName target:target action:action];
}

- (UIButton *)setLeftNavigationItemWithName:(NSString *)name imageName:(NSString *)imageName target:(id)target action:(SEL)action {
    return [self setNavButton:self.leftNavBtn name:name imageName:imageName target:target action:action];
}

- (UIButton *)setRightNavigationItemWithName:(NSString *)name imageName:(NSString *)imageName target:(id)target action:(SEL)action {
    return [self setNavButton:self.rightNavBtn name:name imageName:imageName target:target action:action];
}

- (UIButton *)setNavButton:(UIButton *)button name:(NSString *)name imageName:(NSString *)imageName target:(id)target action:(SEL)action {
    [button setTitle:name forState:UIControlStateNormal];
    
    if (imageName.length > 0) {
        UIImage *image = [[UIImage imageNamed:imageName] imageWithRenderingMode:UIImageRenderingModeAlwaysOriginal];
        if (image) {
            [button setImage:image forState:UIControlStateNormal];
        }
    }
    
    [button addTarget:target action:action forControlEvents:UIControlEventTouchUpInside];
    return button;
}

- (UIButton *)createNavButton {
    UIButton *button = [UIButton buttonWithType:UIButtonTypeCustom];
    [button setTitleColor:self.navTitleColor forState:UIControlStateNormal];
    button.titleLabel.font = self.navRightFont;
    button.titleLabel.adjustsFontSizeToFitWidth = YES;
    return button;
}

@end
