//
//  UIButton+AS.swift
//  AIPlayRingtones
//
//  Created by mini on 2025/5/27.
//

public extension UIButton {
    static func createBtn(title: String? = nil,
                             image: UIImage? = nil,
                             backgroundImage: UIImage? = nil,
                             backgroundColor: UIColor? = nil,
                             font: UIFont? = nil,
                             titleColor: UIColor? = nil,
                             corner:CGFloat = 0,
                             action: (() -> Void)? = nil) -> UIButton {
        let button = UIButton(type: .custom)
        button.setUpBtn(title: title,
                           image: image,
                           backgroundImage: backgroundImage,
                           backgroundColor: backgroundColor,
                           font: font,
                           titleColor: titleColor,
                           corner:corner,
                           action: action)
        return button
    }

    func setUpBtn(title: String? = nil,
                             image: UIImage? = nil,
                             backgroundImage: UIImage? = nil,
                             backgroundColor: UIColor? = nil,
                             font: UIFont? = nil,
                             titleColor: UIColor? = nil,
                             corner:CGFloat = 0,
                             action: (() -> Void)? = nil){

        if let image = image {
            setImage(image, for: .normal)
        }
        if let title = title {
            setTitle(title, for: .normal)
        }
        
        if corner > 0 {
            cornerRadius = corner
        }
   
        if let backgroundImage = backgroundImage {
            setBackgroundImage(backgroundImage, for: .normal)
        }
        if let font = font {
            titleLabel?.font = font
        }

        showsTouchWhenHighlighted = false
        adjustsImageWhenHighlighted = false

        if let titleColor = titleColor {
            setTitleColor(titleColor, for: .normal)
        }
        if let backgroundColor = backgroundColor {
            self.backgroundColor = backgroundColor
        }
    
        if let action = action {
            actionClosure = action
            addTarget(self, action: #selector(buttonTapped), for: .touchUpInside)
        }

    }

    // 按钮点击事件
    @objc private func buttonTapped() {
        actionClosure?()
    }
}
public extension UIButton {
    
    private struct AssociatedKeys {
        static var actionKey = "ASButtonActionKey"
    }
    
    private var actionClosure: (() -> Void)? {
        get {
            return objc_getAssociatedObject(self, &AssociatedKeys.actionKey) as? (() -> Void)
        }
        set {
            objc_setAssociatedObject(self, &AssociatedKeys.actionKey, newValue, .OBJC_ASSOCIATION_RETAIN_NONATOMIC)
        }
    }
    
}


public extension UIButton {
    
    func preventMultipleTaps(delay: Double = 0.75) {
        isEnabled = false
        DispatchQueue.main.asyncAfter(deadline: .now() + delay) {
            self.isEnabled = true
        }
    }
    
}
