//
//  UIColor+AS.m
//  AIPlayRingtones
//
//  Created by mini on 2025/5/27.
//

#import "UIColor+AS.h"
#import <UIKit/UIKit.h>

@implementation UIColor (ASHex)

+ (UIColor *)fromHex:(NSString *)hexString {
    NSString *cleanString = [hexString stringByReplacingOccurrencesOfString:@"#" withString:@""];
    
    if (cleanString.length == 3) {
        cleanString = [NSString stringWithFormat:@"%@%@%@%@%@%@",
                       [cleanString substringWithRange:NSMakeRange(0, 1)],
                       [cleanString substringWithRange:NSMakeRange(0, 1)],
                       [cleanString substringWithRange:NSMakeRange(1, 1)],
                       [cleanString substringWithRange:NSMakeRange(1, 1)],
                       [cleanString substringWithRange:NSMakeRange(2, 1)],
                       [cleanString substringWithRange:NSMakeRange(2, 1)]];
    }
    
    if (cleanString.length == 6) {
        cleanString = [cleanString stringByAppendingString:@"ff"];
    }
    
    unsigned int rgbaValue = 0;
    [[NSScanner scannerWithString:cleanString] scanHexInt:&rgbaValue];
    
    CGFloat red = ((rgbaValue & 0xFF000000) >> 24) / 255.0f;
    CGFloat green = ((rgbaValue & 0x00FF0000) >> 16) / 255.0f;
    CGFloat blue = ((rgbaValue & 0x0000FF00) >> 8) / 255.0f;
    CGFloat alpha = (rgbaValue & 0x000000FF) / 255.0f;
    
    return [UIColor colorWithRed:red green:green blue:blue alpha:alpha];
}

@end


@implementation UIColor (ImageExtension)

- (UIImage *)imageWithSize:(CGSize)size {
    // 确保大小有效
    if (size.width <= 0 || size.height <= 0) {
        return nil;
    }
    
    // 创建绘图上下文
    CGRect rect = CGRectMake(0, 0, size.width, size.height);
    UIGraphicsBeginImageContextWithOptions(rect.size, NO, [UIScreen mainScreen].scale);
    
    // 设置填充颜色并绘制矩形
    CGContextRef context = UIGraphicsGetCurrentContext();
    CGContextSetFillColorWithColor(context, self.CGColor);
    CGContextFillRect(context, rect);
    
    // 获取生成的图片
    UIImage *image = UIGraphicsGetImageFromCurrentImageContext();
    
    // 结束绘图上下文
    UIGraphicsEndImageContext();
    
    return image;
}

@end
