//
//  UIImageView+AS.swift
//  AIPlayRingtones
//
//  Created by mini on 2025/5/28.
//

import Kingfisher

public extension UIImageView {
 static func createImageV(
        image:UIImage? = nil,
        imageName: String? = nil,
        contentMode: UIView.ContentMode = .scaleAspectFit,
        backgroundColor: UIColor = .clear,
        corner: CGFloat = 0.0)
    -> UIImageView {
        let imageView = UIImageView()
                                    
        if let image = image{
            imageView.image = image
        }

        if let imageName = imageName ,imageName.count > 0 {
            imageView.image = UIImage(named: imageName)
        }
        
        imageView.contentMode = contentMode
        imageView.backgroundColor = backgroundColor
        imageView.cornerRadius = corner
        return imageView
    }

    func setImageAsync(urlString: String?,
                     placeholder: UIImage? = nil,
                     contentMode: UIView.ContentMode = .scaleAspectFit,
                 adaptiveMode:Bool = false,
                 backgroundColor: UIColor = .clear,
                      showLoading: Bool = false,
                        progressBlock: ((Float)->Void)? = nil,
                      completion: ((UIImage?) -> Void)? = nil){
        let imageView = self
        imageView.contentMode = contentMode
        imageView.backgroundColor = backgroundColor
        imageView.image = placeholder
        
        guard let urlString = urlString else {
            completion?(nil)
            return
        }
      
        if urlString.isEmpty{
            completion?(nil)
            return
        }
        
        if urlString.contains("http") {
            guard let url = URL(string: urlString) else {
                completion?(nil)
                return
            }
            kf.indicatorType = .none
            imageView.kf.setImage(with: url,
                 placeholder: placeholder,
                     options: nil,
                progressBlock: { receivedSize, totalSize in
                let progress = receivedSize/totalSize
                progressBlock?(Float(progress))
            }){ result in
                
                if let image = try? result.get().image {
                    DispatchQueue.main.asyncAfter(deadline: .now() + 0.1) {
                        completion?(image)
                    }
                }else{
                    completion?(nil)
                }
            }
            
        }else if urlString.contains("/") {
            imageView.image = UIImage(contentsOfFile: urlString.fillCachePath)
            completion?(imageView.image)
        }else {
            if let image = UIImage(named: urlString) {
                imageView.image = image
                completion?(image)
            }
        }
    }
}
