//
//  UIViewController+CustomAlert.m
//  AIPlayRingtones
//
//  Created by mini on 2025/5/27.
//

#import "UIViewController+CustomAlert.h"

@implementation UIViewController (CustomAlert)


- (void)showCustomAlertWithTitle:(NSString *)title
                        message:(NSString *)message
              rightActionString:(NSString *)rightActionString
                  deleteHandler:(void(^)(void))deleteHandler
                  cancelHandler:(void(^)(void))cancelHandler {
    
    UIAlertController *alert = [UIAlertController alertControllerWithTitle:nil message:nil preferredStyle:UIAlertControllerStyleAlert];
    
    // 自定义标题
    if (title.length > 0) {
        NSDictionary *titleAttributes = @{
            NSFontAttributeName: [UIFont boldSystemFontOfSize:20],
            NSForegroundColorAttributeName: [UIColor whiteColor]
        };
        NSAttributedString *attributedTitle = [[NSAttributedString alloc] initWithString:title attributes:titleAttributes];
        [alert setValue:attributedTitle forKey:@"attributedTitle"];
    }
    
    // 自定义消息
    if (message.length > 0) {
        NSDictionary *messageAttributes = @{
            NSFontAttributeName: [UIFont boldSystemFontOfSize:17],
            NSForegroundColorAttributeName: [UIColor whiteColor]
        };
        NSAttributedString *attributedMessage = [[NSAttributedString alloc] initWithString:message attributes:messageAttributes];
        [alert setValue:attributedMessage forKey:@"attributedMessage"];
    }
    
    // 添加取消按钮
    NSString *cancelTitle = NSLocalizedString(@"Cancel", nil);
    UIAlertAction *cancelAction = [UIAlertAction actionWithTitle:cancelTitle
                                                          style:UIAlertActionStyleCancel
                                                        handler:^(UIAlertAction * _Nonnull action) {
        if (cancelHandler) {
            cancelHandler();
        }
    }];
    [alert addAction:cancelAction];
    
    // 添加删除/确认按钮
    NSString *actionTitle = rightActionString ?: NSLocalizedString(@"Delete", nil);
    UIAlertAction *deleteAction = [UIAlertAction actionWithTitle:actionTitle
                                                          style:UIAlertActionStyleDestructive
                                                        handler:^(UIAlertAction * _Nonnull action) {
        if (deleteHandler) {
            deleteHandler();
        }
    }];
    [alert addAction:deleteAction];
    
    // 设置黑暗模式
    if (@available(iOS 13.0, *)) {
        alert.overrideUserInterfaceStyle = UIUserInterfaceStyleDark;
    }
    
    // 显示弹窗
    [self presentViewController:alert animated:YES completion:nil];
}

- (void)showCustomAlertWithMessage:(NSString *)message
                 rightActionString:(NSString *)rightActionString
                     deleteHandler:(void(^)(void))deleteHandler
                     cancelHandler:(void(^)(void))cancelHandler {
    
    [self showCustomAlertWithTitle:nil
                          message:message
                rightActionString:rightActionString
                    deleteHandler:deleteHandler
                    cancelHandler:cancelHandler];
}

@end
