//
//  ASCustomAlert.swift
//  AIPlayRingtones
//
//  Created by mini on 2025/5/28.
//

public class ASCustomAlert {
    
    // MARK: - 配置模型
    public struct AlertConfig {
        public var style: UIUserInterfaceStyle = .dark
        public var message: String
        public var messageColor: UIColor = .black
        public var messageFont: UIFont = .systemFont(ofSize: 14)
        
        public var cancelTitle: String = "取消"
        public var cancelColor: UIColor = .gray
        
        public var confirmTitle: String = "确定"
        public var confirmColor: UIColor = .blue
        
        public var cancelAction: (() -> Void)?
        public var confirmAction: (() -> Void)?
        
        public init(style:UIUserInterfaceStyle = .dark, message: String, messageColor: UIColor, messageFont: UIFont, cancelTitle: String, cancelColor: UIColor, confirmTitle: String, confirmColor: UIColor, cancelAction: ( () -> Void)? = nil, confirmAction: ( () -> Void)? = nil) {
            self.style = style
            self.message = message
            self.messageColor = messageColor
            self.messageFont = messageFont
            self.cancelTitle = cancelTitle
            self.cancelColor = cancelColor
            self.confirmTitle = confirmTitle
            self.confirmColor = confirmColor
            self.cancelAction = cancelAction
            self.confirmAction = confirmAction
        }
    }
    
    // MARK: - 显示弹窗
    public static func show(in viewController: UIViewController, config: AlertConfig) {
        let alert = UIAlertController(title: "", message: "", preferredStyle: .alert)
        let messageAttributed = NSAttributedString(
            string: config.message,
            attributes: [
                .foregroundColor: config.messageColor,
                .font: config.messageFont
            ]
        )
        alert.setValue(messageAttributed, forKey: "attributedMessage")
        let cancelAction = UIAlertAction(title: config.cancelTitle, style: .default) { _ in
            config.cancelAction?()
        }
        cancelAction.setValue(config.cancelColor, forKey: "titleTextColor")
        alert.addAction(cancelAction)
        let confirmAction = UIAlertAction(title: config.confirmTitle, style: .default) { _ in
            config.confirmAction?()
        }
        confirmAction.setValue(config.confirmColor, forKey: "titleTextColor")
        alert.addAction(confirmAction)
        alert.overrideUserInterfaceStyle = config.style
        viewController.present(alert, animated: true)
    }
}
