//
//  ASDynamicContainer.swift
//  AIPlayRingtones
//
//  Created by mini on 2025/5/28.
//

import UIKit
import SnapKit

open class ASDynamicContainer: UIView {
    
    // MARK: - UI Components
    let contentScroller: UIScrollView
    let elementArranger: UIStackView
    
    // MARK: - Configuration Properties
    public var layoutOrientation: NSLayoutConstraint.Axis {
        get { elementArranger.axis }
        set {
            elementArranger.axis = newValue
            refreshLayoutConstraints()
        }
    }
    
    public var elementGap: CGFloat {
        get { elementArranger.spacing }
        set { elementArranger.spacing = newValue }
    }
    
    public var computedHeight: CGFloat {
        contentScroller.contentSize.height
    }
    
    // MARK: - Initialization
    public init(
        orientation: NSLayoutConstraint.Axis = .vertical,
        contentAlignment: UIStackView.Alignment = .leading,
        gapSize: CGFloat = 0
    ) {
        self.contentScroller = UIScrollView()
        self.elementArranger = UIStackView()
        
        super.init(frame: .zero)
        
        configureScroller()
        configureArranger(
            axis: orientation,
            alignment: contentAlignment,
            spacing: gapSize
        )
        assembleInterface()
    }
    
    required public init?(coder: NSCoder) {
        fatalError("Unsupported initialization method")
    }
    
    // MARK: - Configuration Methods
    private func configureScroller() {
        contentScroller.showsVerticalScrollIndicator = false
        contentScroller.showsHorizontalScrollIndicator = false
    }
    
    private func configureArranger(
        axis: NSLayoutConstraint.Axis,
        alignment: UIStackView.Alignment,
        spacing: CGFloat
    ) {
        elementArranger.axis = axis
        elementArranger.alignment = alignment
        elementArranger.spacing = spacing
        elementArranger.distribution = .fill
    }
    
    private func assembleInterface() {
        addSubview(contentScroller)
        contentScroller.snp.makeConstraints { $0.edges.equalToSuperview() }
        
        contentScroller.addSubview(elementArranger)
        refreshLayoutConstraints()
    }
    
    private func refreshLayoutConstraints() {
        elementArranger.snp.remakeConstraints {
            $0.edges.equalToSuperview()
            
            switch layoutOrientation {
            case .vertical:
                $0.width.equalTo(contentScroller)
            case .horizontal:
                $0.height.equalTo(contentScroller)
            @unknown default:
                break
            }
        }
    }
    
    // MARK: - Content Management
    public func embedView(
        _ view: UIView,
        dimension: CGFloat? = nil
    ) {
        elementArranger.addArrangedSubview(view)
        
        view.snp.makeConstraints {
            switch layoutOrientation {
            case .vertical:
                $0.width.equalTo(elementArranger)
                if let dim = dimension { $0.height.equalTo(dim) }
            case .horizontal:
                $0.height.equalTo(elementArranger)
                if let dim = dimension { $0.width.equalTo(dim) }
            @unknown default:
                break
            }
        }
    }
    
    public func appendSpacer(_ length: CGFloat) {
        let spacer = UIView()
        elementArranger.addArrangedSubview(spacer)
        
        spacer.snp.makeConstraints {
            switch layoutOrientation {
            case .vertical:
                $0.width.equalToSuperview()
                $0.height.equalTo(length)
            case .horizontal:
                $0.width.equalTo(length)
                $0.height.equalToSuperview()
            @unknown default:
                break
            }
        }
    }
}

// MARK: - StackView Extension
extension UIStackView {
    public func appendSpacer(_ length: CGFloat) {
        let spacer = UIView()
        self.addArrangedSubview(spacer)
        
        spacer.snp.makeConstraints {
            switch axis {
            case .vertical:
                $0.width.equalToSuperview()
                $0.height.equalTo(length)
            case .horizontal:
                $0.width.equalTo(length)
                $0.height.equalToSuperview()
            @unknown default:
                break
            }
        }
    }
}
