//
//  ASGeneratorLoadingView.swift
//  AIPlayRingtones
//
//  Created by 100Years on 2025/5/18.
//

import Kingfisher

class ASGeneratorLoadingView : ASBaseView {

    
    var isShowBackGeneration:Bool = false {
        didSet{
            self.backgroundGenerateBtn.isHidden = !self.isShowBackGeneration
        }
    }
    
    lazy var cusStackView: UIStackView = {
        let cusStackView = UIStackView()
        cusStackView.axis = .vertical
        cusStackView.distribution = .fill
        cusStackView.alignment = .center
        return cusStackView
    }()
    
    lazy var animatedImageView: AnimatedImageView = {
        let animatedImageView = AnimatedImageView()
        animatedImageView.autoPlayAnimatedImage = false
        if let gifURL = Bundle.main.url(forResource: "rotatingAnimation", withExtension: "gif") {
            animatedImageView.kf.setImage(with: gifURL, options: [.cacheOriginalImage]) { result in
                switch result {
                case .success(let value):
                    print("GIF 加载成功: \(value.source.url?.absoluteString ?? "")")
                case .failure(let error):
                    print("GIF 加载失败: \(error.localizedDescription)")
                }
            }
        }
        
        return animatedImageView
    }()
    
    lazy var timeLabel: UILabel = {
        let textLabel = UILabel.createLabel(font: .font(size: 18,weight: .semibold),textColor: .white,textAlignment: .center)
        textLabel.isHidden = true
        return textLabel
    }()
    
    lazy var progressView: TSRingProgressView = {
        let progressView = TSRingProgressView()
        return progressView
    }()
    
    lazy var textLabel: UILabel = {
        let textLabel = UILabel.createLabel(font: .font(size: 18),textColor: .white,textAlignment: .center,numberOfLines: 0)
        textLabel.isHidden = true
        return textLabel
    }()
    
    lazy var infoLabel: UILabel = {
        let textLabel = UILabel.createLabel(font: .font(size: 14),textColor: .white.withAlphaComponent(0.6),textAlignment: .center,numberOfLines: 0)
        textLabel.isHidden = true
        return textLabel
    }()
    
    lazy var backgroundGenerateBtn: UIButton = {
        let textColor = "#B48EFF".uiColor
        let btn = UIButton.createBtn(title: "Generate in the background".localized,font: .font(size: 16),titleColor: textColor,corner: 24)
        btn.layer.borderColor = textColor.cgColor
        btn.layer.borderWidth = 1.0
        btn.titleLabel?.adjustsFontSizeToFitWidth = true
        btn.isHidden = true
        return btn
    }()
    
    
    override func creatUI() {
        
        contentView.addSubview(cusStackView)
        cusStackView.snp.makeConstraints { make in
            make.edges.equalToSuperview()
        }
        
        setUpGeneratorContentView()
    }
    
    func setUpGeneratorContentView(){
        
        cusStackView.addArrangedSubview(animatedImageView)
        animatedImageView.snp.makeConstraints { make in
            make.top.equalTo(0)
            make.width.height.equalTo(120)
        }
        
        cusStackView.appendSpacer(12.0)
        
        cusStackView.addArrangedSubview(timeLabel)
        timeLabel.snp.makeConstraints { make in
            make.leading.equalTo(kTextLeading)
            make.trailing.equalTo(-kTextLeading)
        }
        
        cusStackView.appendSpacer(12.0)
        
        cusStackView.addArrangedSubview(progressView)
        progressView.snp.makeConstraints { make in
            make.leading.equalTo(40)
            make.trailing.equalTo(-40)
        }
        
        cusStackView.appendSpacer(8.0)
        cusStackView.addArrangedSubview(textLabel)
        textLabel.snp.makeConstraints { make in
            make.leading.equalTo(kTextLeading)
            make.trailing.equalTo(-kTextLeading)
        }
        
        cusStackView.appendSpacer(8.0)
        
        cusStackView.addArrangedSubview(infoLabel)
        infoLabel.snp.makeConstraints { make in
            make.leading.equalTo(kTextLeading)
            make.trailing.equalTo(-kTextLeading)
        }
        
        cusStackView.appendSpacer(20.0)
        
        cusStackView.addArrangedSubview(backgroundGenerateBtn)
        backgroundGenerateBtn.snp.makeConstraints { make in
            make.width.lessThanOrEqualTo(k_ScreenWidth - 64)
            make.width.greaterThanOrEqualTo(250)
            make.height.equalTo(48)
            make.bottom.equalTo(0)
        }

    }

}

extension ASGeneratorLoadingView {
    
    
    func startAnimation() {
        kDelayMainShort {
            self.animatedImageView.startAnimating()
        }
    }

    func stopAnimation() {
        self.animatedImageView.stopAnimating()
    }
    
    func setText(time:String,info:String) {
        timeLabel.text = time
        infoLabel.text = info
        
        timeLabel.isHidden = time.isEmpty
        infoLabel.isHidden = info.isEmpty
    }
    
    func setProgressText(progress:Float, text:String) {
        textLabel.text = text
        textLabel.isHidden = text.isEmpty
        progressView.setProgress(progress: progress)
    }

}
