//
//  ASSaveResultManager.swift
//  AIPlayRingtones
//
//  Created by mini on 2025/5/28.
//

public let kSuccessManager = ASSaveResultManager.shared

open class ASSaveResultManager {
    
    static let shared = ASSaveResultManager()
    
    public var completionHandler: (() -> Void)?
    
    private lazy var messageLabel: UILabel = {
        let label = UILabel()
        label.textColor = .white
        label.text = "Save Successfully".localized
        label.font = .systemFont(ofSize: 14)
        return label
    }()
    
    private lazy var containerView: UIView = {
        return createNotificationView()
    }()
    
    private lazy var actionButton: UIView = {
        let color = "4FEA9D".uiColor
        let button = UIButton.create(
            title: "View".localized,
            backgroundColor: color.withAlphaComponent(0.1),
            font: .systemFont(ofSize: 12),
            titleColor: color,
            cornerRadius: 14
        ) { [weak self] in
            self?.handleButtonAction()
        }
        button.contentEdgeInsets = UIEdgeInsets(top: 0, left: 10, bottom: 0, right: 10)
        return button
    }()
    
    private func createNotificationView() -> UIView {
        let backgroundView = UIView(frame: CGRect(x: 0, y: 0, width: 288, height: 48))
        backgroundView.backgroundColor = .clear
        backgroundView.layer.configureShadow(
            color: .black,
            offset: CGSize(width: 0, height: 2),
            opacity: 0.1
        )
        
        let contentView = UIView()
        contentView.backgroundColor = "333333".uiColor
        contentView.layer.cornerRadius = 8
        contentView.clipsToBounds = true
        
        backgroundView.addSubview(contentView)
        contentView.snp.makeConstraints { $0.edges.equalToSuperview() }
        
        let icon = UIImageView(image: UIImage(named: "success_icon"))
        backgroundView.addSubview(icon)
        icon.snp.makeConstraints {
            $0.size.equalTo(24)
            $0.centerY.equalToSuperview()
            $0.leading.equalTo(12)
        }
        
        backgroundView.addSubview(actionButton)
        backgroundView.addSubview(messageLabel)
        
        actionButton.snp.makeConstraints {
            $0.width.equalTo(actionButton.intrinsicContentSize.width)
            $0.height.equalTo(28)
            $0.trailing.equalTo(-8)
            $0.centerY.equalToSuperview()
        }
        
        messageLabel.snp.makeConstraints {
            $0.leading.equalTo(icon.snp.trailing).offset(8)
            $0.trailing.equalTo(actionButton.snp.leading).offset(-8)
            $0.centerY.equalToSuperview()
        }
        
        return backgroundView
    }
    
    public func calculateBottomPosition(from topPosition: CGFloat) -> CGFloat {
        let position = -(UIScreen.main.bounds.height - 48 - topPosition)
        print("Calculated position: \(position)")
        return position
    }
    
    public func displayNotification(
        in view: UIView,
        message: String = "Save Successfully".localized,
        duration: Double = 2.0,
        position: CGFloat = -112,
        showAction: Bool = true,
        handler: (() -> Void)? = nil
    ) {
        self.completionHandler = handler
        
        DispatchQueue.main.async {
            self.messageLabel.text = message
            self.actionButton.isHidden = !showAction
            view.addSubview(self.containerView)
            
            self.containerView.snp.remakeConstraints {
                $0.width.greaterThanOrEqualTo(288)
                $0.width.lessThanOrEqualTo(UIScreen.main.bounds.width - 32)
                $0.height.equalTo(48)
                $0.centerX.equalToSuperview()
                $0.bottom.equalTo(position)
            }
            
            DispatchQueue.main.asyncAfter(deadline: .now() + duration) {
                self.containerView.removeFromSuperview()
            }
        }
    }
    
    private func handleButtonAction() {
        if let handler = completionHandler {
            handler()
        } else {
            if let url = URL(string: "photos-redirect://"),
               UIApplication.shared.canOpenURL(url) {
                UIApplication.shared.open(url, options: [:])
            }
        }
        DispatchQueue.main.async {
            self.containerView.removeFromSuperview()
        }
    }
}

// MARK: - Extensions
private extension CALayer {
    func configureShadow(color: UIColor, offset: CGSize, opacity: Float) {
        shadowColor = color.cgColor
        shadowOffset = offset
        shadowOpacity = opacity
    }
}

private extension UIButton {
    static func create(
        title: String,
        backgroundColor: UIColor,
        font: UIFont,
        titleColor: UIColor,
        cornerRadius: CGFloat,
        action: @escaping () -> Void
    ) -> UIButton {
        let button = UIButton()
        button.setTitle(title, for: .normal)
        button.backgroundColor = backgroundColor
        button.titleLabel?.font = font
        button.setTitleColor(titleColor, for: .normal)
        button.layer.cornerRadius = cornerRadius
        button.addAction(for: .touchUpInside, action)
        return button
    }
}

private extension UIControl {
    func addAction(for event: UIControl.Event, _ closure: @escaping () -> Void) {
        let action = UIAction { _ in closure() }
        addAction(action, for: event)
    }
}
