//
//  ASTLLabel.swift
//  AIPlayRingtones
//
//  Created by mini on 2025/5/28.
//

open class ASTLLabel: UILabel {
    
    // 设置内间距
    var textInsets: UIEdgeInsets = .zero {
        didSet {
            setNeedsDisplay()
        }
    }
    
    open override var intrinsicContentSize: CGSize {
        // 根据内容大小和内间距调整固有内容大小
        guard let text = text else { return .zero }
        let textSize = (text as NSString).boundingRect(
            with: CGSize(width: bounds.width - textInsets.left - textInsets.right, height: .greatestFiniteMagnitude),
            options: [.usesLineFragmentOrigin, .usesFontLeading],
            attributes: [.font: font ?? UIFont.systemFont(ofSize: 17)],
            context: nil
        ).size
        return CGSize(width: textSize.width + textInsets.left + textInsets.right, height: textSize.height + textInsets.top + textInsets.bottom)
    }
    
    open override func drawText(in rect: CGRect) {
        // 根据内间距调整绘制区域
        let insetRect = rect.inset(by: textInsets)
        guard let text = text else { return }
        let textSize = (text as NSString).boundingRect(
            with: CGSize(width: insetRect.width, height: .greatestFiniteMagnitude),
            options: [.usesLineFragmentOrigin, .usesFontLeading],
            attributes: [.font: font ?? UIFont.systemFont(ofSize: 17)],
            context: nil
        ).size
        
        // 重新定义绘制区域，使文本始终靠左上角
        var adjustedRect = insetRect
        adjustedRect.size.height = min(textSize.height, insetRect.height)
        
        // 调用父类的绘制方法
        super.drawText(in: adjustedRect)
    }

}
