//
//  ASActionInfoModel.swift
//  AIPlayRingtones
//
//  Created by 100Years on 2025/5/16.
//

import ObjectMapper
class ASActionInfoModel: ASBaseModel {
    
    enum ModelType:Int {
        case normal = 0
        case example = 1
        case ttr = 2    //文生铃声
        case rtr = 3    //铃声生铃声
    }
    
    enum ActionStatus:String ,Equatable {
        case success = "success"//成功
        case pending = "pending"//等待
        case running = "running"//运行
        case failed = "failed"//失败
        
        static func from(_ string: String) -> ActionStatus {
            return ActionStatus(rawValue: string) ?? .failed
        }
    }
    var modelType:ModelType = .normal
    var id:Int = 0
    var actionType:String = ""
    var comments:String = ""
    var request:ASActionRequestModel = ASActionRequestModel()
    var response:ASActionResponseModel = ASActionResponseModel()
    var createdTimestamp:Int = 0
    var status:String = ""
    var costTime:Int = 0
    var percent:Float = 0.0
    var actionStatus:ActionStatus = .failed
    

    var ringSavePath:String = ""
    
    var uuid:String = UUID().uuidString
    override func mapping(map: ObjectMapper.Map) {
        modelType           <- map["modelType"]
        id           <- map["id"]
        actionType   <- map["actionType"]
        comments     <- map["comments"]
        request      <- (map["request"],JsonStringTransform<ASActionRequestModel>())
        response           <- (map["response"],JsonStringTransform<ASActionResponseModel>())
        createdTimestamp   <- map["createdTimestamp"]
        status     <- map["status"]
        costTime      <- map["costTime"]
        percent     <- map["percent"]
        actionStatus      <- map["actionStatus"]
        actionStatus = ActionStatus.from(status)
        
        ringSavePath      <- map["ringSavePath"]
        
        uuid      <- map["uuid"]
    }
}

extension ASActionInfoModel {

    var isExpired:Bool{
        if request.uploadUrl.count == 0 {
            return true
        }
        
        if request.uploadTimestamp <= 0 {
            return true
        }
        //86400=24小时
        if Date.timestampInt - request.uploadTimestamp > 86400 {
            return true
        }
        
        return false
    }
    
    
    var localURL: URL? {
        if modelType == .example {
            if let url = Bundle.main.url(forResource: response.musicUrl, withExtension: nil) {
                return url
            }
        }
        
        if !ringSavePath.isEmpty {
            return ringSavePath.fillDocumentURL
        }
        
        return nil
    }
    
    

    
}

class ASActionRequestModel : ASBaseModel {

    var width:Int = 0
    var height:Int = 0
    
    //生成音乐用的
    var duration:Int = 0
    var prompt:String = ""
    var inputText:String = ""  //用户自己输入的内容
    
    var uploadUrl:String = ""
    var uploadTimestamp:Int = 0 //上传的时间戳
    
    override func mapping(map: ObjectMapper.Map) {
        prompt              <- map["prompt"]
        inputText          <- map["inputText"]
        duration            <- map["duration"]
        
        uploadUrl            <- map["uploadUrl"]
        uploadTimestamp   <- map["uploadTimestamp"]
    }
}

class ASActionResponseModel : ASBaseModel {
    var code:Int = 0
    var vip:Bool = false
    
    
    //生成音乐用的
    var coverUrl:String = ""
    var title:String = ""
    var musicUrl:String = ""
    //自己计算的
    var duration:Int = 0
    var size:Int = 0
    override func mapping(map: ObjectMapper.Map) {
        vip                 <- map["vip"]
        code                <- map["code"]
        
        coverUrl           <- map["coverUrl"]
        title              <- map["title"]
        musicUrl           <- map["musicUrl"]
        
        duration           <- map["duration"]
        size               <- map["size"]
    }
    
    //获取生成错误 code 对应的文案
    var codeErrorMsg:String {
        ASErrorCode.message(for: code)
    }
    //敏感错误
    var sensitiveError:Bool {
        return ASErrorCode.isContentFlagged(code)
    }
}

