//
//  ASDBActionInfoModel.swift
//  AIPlayRingtones
//
//  Created by 100Years on 2025/5/16.
//


import RealmSwift

class TSDBActionInfoModel: Object {
 
    @Persisted(primaryKey: true) var primaryKey: String = UUID().uuidString
    @Persisted var createdAt: Date = Date()
    
    @Persisted var modelType:Int = 0
    @Persisted var id:Int = 0
    @Persisted var actionType:String = ""
    @Persisted var comments:String = ""
    @Persisted var request:TSDBActionRequestModel?
    @Persisted var response:TSDBActionResponseModel?
    @Persisted var createdTimestamp:Int = 0
    @Persisted var status:String = ""   //success,pending,running,failed
    @Persisted var costTime:Int = 0
    @Persisted var percent:Float = 0.0
    
    @Persisted var videoThumbnailPath:String = ""
    @Persisted var videoPath:String = ""
    @Persisted var uuid:String = UUID().uuidString
    
    static func createDBModel(actionInfoModel:ASActionInfoModel) -> TSDBActionInfoModel{
        let dbModel = TSDBActionInfoModel()
        dbModel.saveData(infoModel: actionInfoModel)
        return dbModel
    }
    
    func saveData(infoModel:ASActionInfoModel) {
        self.modelType = infoModel.modelType.rawValue
        self.id = infoModel.id
        self.actionType = infoModel.actionType
        self.comments = infoModel.comments

        self.createdTimestamp = infoModel.createdTimestamp
        self.status = infoModel.status
        self.costTime = infoModel.costTime
        self.percent = infoModel.percent

        self.uuid = infoModel.uuid
        
        self.request = TSDBActionRequestModel.createDBModel(requestModel: infoModel.request)
        self.response = TSDBActionResponseModel.createDBModel(responseModel: infoModel.response)
        
    }
    
    func getModel()->ASActionInfoModel{
        let infoModel = ASActionInfoModel()
        
        infoModel.modelType = ASActionInfoModel.ModelType.init(rawValue: self.modelType) ?? .normal
        infoModel.id = self.id
        infoModel.actionType = self.actionType
        infoModel.comments = self.comments

        infoModel.createdTimestamp = self.createdTimestamp
        infoModel.status = self.status
        infoModel.costTime = self.costTime
        infoModel.percent = self.percent
   
        infoModel.uuid = self.uuid

        if let request = self.request {
            infoModel.request = request.getModel()
        }
        
        if let response = self.response {
            infoModel.response = response.getModel()
        }
        
        infoModel.actionStatus = ASActionInfoModel.ActionStatus.from(infoModel.status)
        return infoModel
    }
    
    
    var isResult:Bool {
        if status.count > 0 {
            if status == "pending" ||
               status == "running"
            {
                return false
            }
        }
        return true
    }

}



class TSDBActionRequestModel : Object {
    
    @Persisted(primaryKey: true) var primaryKey: String = UUID().uuidString
    @Persisted var createdAt: Date = Date()
    
    @Persisted var width:Int = 0
    @Persisted var height:Int = 0
    
    @Persisted var prompt:String = ""
    @Persisted var inputText:String = ""  //用户自己输入的内容
    @Persisted var duration:Int = 5 //音乐秒数

    
    @Persisted var uploadUrl:String = ""
    @Persisted var uploadTimestamp:Int = 0
    
    
    static func createDBModel(requestModel:ASActionRequestModel) -> TSDBActionRequestModel{
        let dbModel = TSDBActionRequestModel()
        dbModel.saveData(requestModel: requestModel)
        return dbModel
    }
    
    func saveData(requestModel:ASActionRequestModel) {
        self.prompt = requestModel.prompt
        self.inputText = requestModel.inputText
        self.duration = requestModel.duration
        
        self.width = requestModel.width
        self.height = requestModel.height
        
        self.uploadUrl = requestModel.uploadUrl
        self.uploadTimestamp = requestModel.uploadTimestamp
    }
    
    
    func getModel()->ASActionRequestModel{
        let model = ASActionRequestModel()
        model.prompt = self.prompt
        model.inputText = self.inputText
        model.duration = self.duration
        
        model.width = self.width
        model.height = self.height
        
        model.uploadUrl = self.uploadUrl
        model.uploadTimestamp = self.uploadTimestamp
        
        return model
    }

}

class TSDBActionResponseModel : Object {
    @Persisted(primaryKey: true) var primaryKey: String = UUID().uuidString
    @Persisted var createdAt: Date = Date()

    @Persisted var code:Int = 0
    @Persisted var vip:Bool = false
    
    
    //生成音乐用的
    @Persisted var coverUrl:String = ""
    @Persisted var title:String = ""
    @Persisted var musicUrl:String = ""
    @Persisted var duration:Int = 0 //音乐秒数
    @Persisted var size:Int = 0 //音乐大小
    
    static func createDBModel(responseModel:ASActionResponseModel) -> TSDBActionResponseModel{
        let dbModel = TSDBActionResponseModel()
        dbModel.saveData(responseModel: responseModel)
        return dbModel
    }
    
    func saveData(responseModel:ASActionResponseModel) {
        self.musicUrl = responseModel.musicUrl
        self.coverUrl = responseModel.coverUrl
        self.title = responseModel.title
        self.duration = responseModel.duration
        self.size = responseModel.size
        
        self.code = responseModel.code
        self.vip = responseModel.vip
    }
    
    
    func getModel()->ASActionResponseModel{
        let model = ASActionResponseModel()
        
        model.coverUrl = self.coverUrl
        model.title = self.title
        model.musicUrl = self.musicUrl
        model.duration = self.duration
        model.size = self.size
        
        model.code = self.code
        model.vip = self.vip
        return model
    }
    
}
