//
//  ASDBHistoryManager.swift
//  AIPlayRingtones
//
//  Created by 100Years on 2025/5/16.
//


import RealmSwift
import ObjectMapper

// 1. 定义历史记录类型枚举
enum ASDBHistoryType: String,CaseIterable {
//    case texToRing = "texToRingHistoryListString"  //图生图
//    case ringToRing = "ringToRingHistoryListString"       //文生图
    case ring = "ringHistoryListString"       //铃声
}


//MARK: TSDBAIChatList - 用于存储会话及消息列表
class ASDBHistory: Object {
    @Persisted(primaryKey: true) var primaryKey: String = ""
    @Persisted var listModels = List<TSDBActionInfoModel>()
    
    var type: ASDBHistoryType {
        get { ASDBHistoryType(rawValue: primaryKey) ?? .ring }
        set { primaryKey = newValue.rawValue }
    }
    
    convenience init(type: ASDBHistoryType) {
        self.init()
        self.primaryKey = type.rawValue
    }
    

    func getModelList() -> [ASActionInfoModel] {
        var msgModel:[ASActionInfoModel] = []
        for msgDBModel in listModels {
            msgModel.append(msgDBModel.getModel())
        }
        return msgModel
    }
    
    
    func getModelList(completion:@escaping ([ASActionInfoModel])->Void){
        let frozenList = self.listModels.freeze()
        DispatchQueue.global(qos: .userInitiated).async {
            var msgModel:[ASActionInfoModel] = []
            for msgDBModel in frozenList {
                msgModel.append(msgDBModel.getModel())
            }
            DispatchQueue.main.async {
                completion(msgModel)
            }
        }
    }
    
    func getModelList(count:Int) -> [ASActionInfoModel] {
        let listModels = Array(listModels.prefix(count))
        var msgModel:[ASActionInfoModel] = []
        for msgDBModel in listModels {
            msgModel.append(msgDBModel.getModel())
        }
        return msgModel
    }
    
    func delete() {
        ASRMShared.delete(self)
    }
    
    func deleteListModel(id:Int) {
        ASRMShared.writeThread {
            if let index = listModels.firstIndex(where: { $0.id == id }) {
                listModels.remove(at: index)
                logPrint("listModels.remove(at: \(index))")
            }
        }
    }
    
    static func deleteAll() {
        do {
            let realm = try Realm()
            try realm.write {
                let allPersons = realm.objects(ASDBHistory.self)
                realm.delete(allPersons)
            }
        } catch {
            logPrint("删除 TSDBPTPHistory 模型数据时出错: \(error)")
        }
    }
    
    func updateData(_ actionInfoModel:ASActionInfoModel,id:Int? = nil){
            let dbModel = TSDBActionInfoModel.createDBModel(actionInfoModel: actionInfoModel)
            var replaceID = dbModel.id
            if let id = id {
                replaceID = id
            }
            
            let frozenList = self.listModels.freeze()
            if let index = frozenList.firstIndex(where: { $0.id == replaceID }) {
                ASRMShared.writeThread {
                    listModels[index] = dbModel// 如果找到，替换该元素
                }
            } else {
                print("Thread.current insert1=\(Thread.current)")
                ASRMShared.writeThread {
                    listModels.insert(dbModel, at: 0)// 如果没有找到，添加到末尾
                    print("Thread.current insert2=\(Thread.current)")
                }
            }
    }
    
    func updateDatas(_ actionInfoModels:[ASActionInfoModel]){
        for actionInfoModel in actionInfoModels {
            updateData(actionInfoModel)
        }
    }

    
    func addDatas(_ actionInfoModels:[ASActionInfoModel]){
        for actionInfoModel in actionInfoModels {
            let dbModel = TSDBActionInfoModel.createDBModel(actionInfoModel: actionInfoModel)
            ASRMShared.writeThread {
                listModels.append(dbModel)
            }
        }
    }
}


extension ASRealmManager {
    func getDBHistory(type:ASDBHistoryType) -> ASDBHistory {
        let predicate = NSPredicate(format: "primaryKey == %@",type.rawValue)
        let historys = ASRMShared.realm.objects(ASDBHistory.self).filter(predicate)
        if let history = historys.first {
            return history
        }else {
            let dbHistory = ASDBHistory(type: type)
            ASRMShared.update(dbHistory)
            return dbHistory
        }
    }
    
    func createExampleModel(id:Int,title:String,musicName:String,duration:Int)->ASActionInfoModel{
        let model = ASActionInfoModel()
        model.id = id
        model.modelType = .example
        model.request.prompt = "Example"
        model.request.inputText = "Example"
        model.request.width = 330
        model.request.height = 440
        
        model.response.coverUrl = ""
        model.response.title = title
        model.response.musicUrl = musicName
        model.response.duration = duration
        model.status = "success"
        return model
    }

    
    //铃声
    var ringDBHistory:ASDBHistory {
        let ptpHistory = getDBHistory(type: ASDBHistoryType.ring)
        
        if ptpHistory.listModels.count == 0,UserDefaults.standard.string(forKey: "insertRingExampleData") == nil {
            let id = Date.timestampInt
            ptpHistory.updateDatas([
                createExampleModel(id:id,title: "Dance" ,musicName: "Dance.mp3",duration: 29),
                createExampleModel(id:id+1,title: "Gentle" ,musicName: "Gentle.mp3",duration: 42)
            ])
            UserDefaults.standard.set("1", forKey: "insertRingExampleData")
            UserDefaults.standard.synchronize()
        }
        
        return ptpHistory
    }
    

}

