//
//  ASRealmManager.swift
//  AIPlayRingtones
//
//  Created by 100Years on 2025/5/16.
//

import RealmSwift

let ASRMShared = ASRealmManager.shared

class ASRealmManager {
    static let shared = ASRealmManager()
    private init() {
        /*设置新的版本号
         1.0 ->1
         **/
   
        let newSchemaVersion: UInt64 = 1
        // 获取默认配置
        var config = Realm.Configuration.defaultConfiguration
        // 设置新版本号
        config.schemaVersion = newSchemaVersion
        // 设置迁移块
        config.migrationBlock = { migration, oldSchemaVersion in
            if oldSchemaVersion < newSchemaVersion {
                // 执行迁移操作
            }
        }
        // 将修改后的配置设置为默认配置
        Realm.Configuration.defaultConfiguration = config
        logPrint("Realm 数据库已成功打开，版本号: \(newSchemaVersion)")
    }
    
    var realm:Realm {
        return try! Realm()
    }


    // 创建数据
    func create<T: Object>(_ object: T) {
        do {
            try realm.write {
                realm.add(object,update: .modified)
            }
        } catch {
            print("Failed to create object: \(error)")
        }
    }

    // 读取数据
    func read<T: Object>(_ type: T.Type) -> Results<T> {
        return realm.objects(type)
    }


    func update<T: Object>(_ object: T) {
        do {
            try Realm().write {
                try Realm().add(object,update: .modified)
            }
        } catch {
            print("Failed to update object: \(error)")
        }
    }

    // 删除数据
    func delete<T: Object>(_ object: T) {
        do {
      
            try Realm().write {
                try Realm().delete(object)
            }
        } catch {
            print("Failed to delete object: \(error)")
        }
    }
    
    
    func writeThread(wt: ()->Void) {
        do {
            try Realm().write {
                wt()
            }
        } catch {
            print("Failed to writeThread: \(error)")
        }
        
    }
}
