//
//  ASBaseOperationQueue.swift
//  AIPlayRingtones
//
//  Created by 100Years on 2025/5/16.
//



import Foundation
import Combine



class ASBaseOperationQueue {
    
    private(set) var queue: OperationQueue = OperationQueue()
    
    private var activeOperations:[String:ASBaseOperation] = [:]
    // 存储每个操作的 AnyCancellable
    private var cancellables: [String: AnyCancellable] = [:]
    // 存储 KVO 观察者
    private var operationCountObservation: NSKeyValueObservation?
    
    var isAvailability:Bool{
        if queue.operationCount < queue.maxConcurrentOperationCount {
            return true
        }
        return false
    }
    
    weak var latestOperation:ASBaseOperation? //最新的任务
    
    init(maxConcurrentOperationCount: Int = 1) {
        queue.maxConcurrentOperationCount = maxConcurrentOperationCount
        logPrint("ASBaseOperationQueue operationCountObservation")
        // 监听 operationCount 的变化
           operationCountObservation = queue.observe(\.operationCount, options: [.new]) { [weak self] (queue, change) in
               guard let _ = self else { return }
               if let _ = change.newValue {
                   NotificationCenter.default.post(name: .kBaseOperationQueueCountChanged, object: nil, userInfo: ["ASBaseOperationQueue":self])
               }
           }
    }

    func creatOperation(uuid: String, type: ASBaseOperation.Type) -> ASBaseOperation {
        if let existingOperation = activeOperations[uuid] {
            return existingOperation
        } else {
            // 使用 type 参数创建操作
            let operation = type.init(uuid: uuid)
            activeOperations[uuid] = operation
            queue.addOperation(operation)
            logPrint("ASBaseOperationQueue addOperation \(operation)")
            cancellables[uuid] = operation.$operationStatePblished.sink { [weak self] state in
                guard let self = self else { return }
                
                switch state {
                case .finished(let finished):
                    if finished == true {
                        clearOperationsData(uuid: uuid)
                    }
                case .cancelled(let cancelled):
                    if cancelled == true {
                        clearOperationsData(uuid: uuid)
                    }
                default: break
                }
                
                logPrint("ASBaseOperationQueue $operationStatePblished =\(state)")
            }
            latestOperation = operation
            return operation
        }
    }
    
    func findOperation<T: ASBaseOperation>(uuid: String) -> T? {
        if let existingOperation = activeOperations[uuid] as? T {
            return existingOperation
        }
        return nil
    }
    
    /// 清理所有下载任务
    func cancelAllOperations() {
        queue.cancelAllOperations()
        activeOperations = [:]
    }
    
    /// 清理某个下载任务
    /// - Parameter url: 下载任务的 URL
    func cancelOperations(uuid: String) {
        if let operation = activeOperations[uuid] {
            operation.cancel()
            clearOperationsData(uuid: uuid)
        }
    }

    func clearOperationsData(uuid: String) {
        logPrint("ASBaseOperationQueue cancelOperations activeOperations 前=\(activeOperations)")
        logPrint("ASBaseOperationQueue cancelOperations cancellables 前=\(cancellables)")
        activeOperations.removeValue(forKey: uuid)
        cancellables.removeValue(forKey: uuid)
        logPrint("ASBaseOperationQueue cancelOperations activeOperations 后=\(activeOperations)")
        logPrint("ASBaseOperationQueue cancelOperations cancellables 后=\(cancellables)")
    }
    func waitUntilAllOperationsAreFinished() {
        queue.waitUntilAllOperationsAreFinished()
    }

}
