//
//  ASGeneratePosterOperation.swift
//  AIPlayRingtones
//
//  Created by 100Years on 2025/5/16.
//

import Combine
import Alamofire
import ObjectMapper
import Kingfisher

class ASGenerateTextToRingOperationQueue: ASGenerateBaseOperationQueue {
    static let shared:ASGenerateTextToRingOperationQueue = ASGenerateTextToRingOperationQueue()

    func creatOperation(uuid: String) -> ASGenerateTextToRingOperation {
        let operation = super.creatOperation(uuid: uuid, type: ASGenerateTextToRingOperation.self)
        handleStateDatauPblished(uuid: uuid, generateOperation: operation as! ASGenerateBaseOperation, notificationName: .kGenerateTTROperationChanged)
        return operation as! ASGenerateTextToRingOperation
    }
    
    override func getUUIDData(uuid:String)->(ASProgressState,ASActionInfoModel?){
        if let PosterOperation = ASGenerateTextToRingOperationQueue.shared.findOperation(uuid: uuid) as? ASGenerateTextToRingOperation {
            logPrint("ASBaseOperation stateDatauPblished 发送 = \(PosterOperation.stateDatauPblished)")
            return (PosterOperation.stateDatauPblished.0,PosterOperation.currentActionInfoModel)
        }
        return (.none,ASActionInfoModel())
    }
    
}

class ASGenerateTextToRingOperation: ASGenerateBaseOperation , @unchecked Sendable{
    
    //是否展示成功后的 View 提醒
    public var isShowSuccessView:Bool = false
    
    override var actionInfoDict:[String:Any]{
        return [
            "actionType":"image_create",
            "comments": "Success",
            "costTime":9,
            "createdTimestamp":1742183242,
            "id":2449,
            "percent":1,
            "request":"{\"prompt\": \"Traditional Chinese ink painting style phoenix soaring through misty mountain peaks, dynamic black brushstrokes with crimson accents on rice paper texture, Googie architecture space station, 1950s atomic age aesthetic, curved aluminum panels, starburst patterns, glass dome observatory --edge_threshold 0.5 --atomic_age_style 0.8 --chrome_reflection 1.2\", \"width\": 800, \"height\": 1440, \"countryCode\": \"FR\"}",
            "response": "{\"resultUrl\": \"https://be-aigc.s3-accelerate.amazonaws.com/4c946f78-b1e7-4ffe-ba18-fff26b10178c.png\"}",
            "status":"success"
        ]
    }

    override func replaceSaveInfoModel(model:ASActionInfoModel){
        model.uuid = uuid
        model.request.uploadTimestamp = currentActionInfoModel.request.uploadTimestamp
        model.modelType = currentActionInfoModel.modelType
        model.id = currentActionInfoModel.id
        if isSaveProcessToDB {
            saveDataDB()
        }

        currentActionInfoModel = model
        logPrint("model actionStatus 发出=\(model.actionStatus)")
        currentActionInfoModelChanged?(currentActionInfoModel)
    }

    override func backstageGeneration(){
        isSaveProcessToDB = true
        isShowSuccessView = true
    }
    
    override func saveDataDB() {
        if currentActionInfoModel.id == 0 {
            return
        }
        logPrint("saveDataDB ASRMShared.ringDBHistory.listModels.count = \(ASRMShared.ringDBHistory.listModels.count),currentActionInfoModel.id = \(currentActionInfoModel.id)")
        ASRMShared.ringDBHistory.updateData(currentActionInfoModel,id: currentActionInfoModel.id)
        logPrint("saveDataDB ASRMShared.ringDBHistory.listModels.count = \(ASRMShared.ringDBHistory.listModels.count)")
    }
    override func handleGenerateSuccess() {
        //只要是生成成功,就一定保存数据,且消耗次数
//        kPurchaseDefault.useOnceForFree(type: .picToPic)
        saveDataDB()
        
        if isShowSuccessView == false {
            return
        }
        
        
        guard let window = ASWindTool.getKeyWindow() else {
            logPrint("getKeyWindow nil")
            return
        }
        
        guard let rootVC = ASWindTool.topViewController() else {
            logPrint("handleGenerateSuccess topViewController nil")
            return
        }
        
        let copyModel = self.currentActionInfoModel.copy()
        if let cyModel = copyModel as? ASActionInfoModel {
            let topY = k_Nav_Height+10
//            logPrint("topY=\(topY)")
            AudioServicesPlaySystemSound(1520)
            kSuccessManager.displayNotification(in: window,message: "Successfully generated".localized,duration:5.0,position: kSuccessManager.calculateBottomPosition(from: topY)){
                let gennerateVC = ASRingGeneratorVC(generateStyleModel: ASGenerateStyleModel(),infoModel: cyModel) { model in }
                gennerateVC.modalPresentationStyle = .overFullScreen
                gennerateVC.modalTransitionStyle = .crossDissolve
                rootVC.present(gennerateVC, animated: true)
            }
        }else{
            logPrint("copyModel as? ASActionInfoModel error")
        }
    }
    
    
    var creatRequest:Request?
    
    func createActionInfoModel(generateStyleModel:ASGenerateStyleModel) -> ASActionInfoModel{

        let infoModel = ASActionInfoModel()
        infoModel.id = Date.timestampInt
        infoModel.request = ASActionRequestModel()
        infoModel.request.uploadUrl = generateStyleModel.upLoadUrl ?? ""
        infoModel.request.uploadTimestamp = Date.timestampInt
        
        infoModel.request.inputText = generateStyleModel.inputText
        infoModel.request.duration = generateStyleModel.duration
        infoModel.request.prompt = generateStyleModel.prompt
        
        infoModel.response.duration = generateStyleModel.duration

        if generateStyleModel.generateType == .ringToRing {
            infoModel.modelType = .rtr
        }else if generateStyleModel.generateType == .textToRing {
            infoModel.modelType = .ttr
        }
        
        return infoModel
    }
    
    //测试接口
//    func creatRing(oldModel:ASActionInfoModel,complete:@escaping (Bool)->Void) {
//        
//        initializeActionInfoModel(oldModel: oldModel)
//        
//        generatingProgress = 0
//        stopNetwork = false
//        
//        currentActionInfoModel.status = "running"
//        currentActionInfoModel.actionStatus = .running
//        currentActionInfoModel.percent = 0
//        replaceSaveInfoModel(model: currentActionInfoModel)
//        
////        stateDatauPblished = (.progress(0.0,generating(progress: 0.0)),currentActionInfoModel)
// 
//        let infoModel = ASActionInfoModel(JSONString: "{\"id\":54914,\"actionType\":\"music_create\",\"request\":\"{\\\"prompt\\\": \\\"Dynamic ringtone, memorable hook, balanced highs and lows, versatile for any phone, Generate a Folk ringtone with a fingerpicked acoustic guitar melody and light percussion. Use a BPM of 80-90 for a warm, earthy feel.\\\", \\\"duration\\\": 5, \\\"audioUrl\\\": null, \\\"countryCode\\\": \\\"CN\\\"}\",\"response\":\"{\\\"title\\\": \\\"Whispering Pines Melody Whispering Pines Melody Whispering Pines Melody Whispering Pines Melody\\\", \\\"musicUrl\\\": \\\"https://be-aigc.s3-accelerate.amazonaws.com/b6651963-c501-4c91-a424-fc2bec0e60ad.wav\\\"}\",\"createdTimestamp\":1747660794,\"status\":\"success\",\"costTime\":7,\"estimateCost\":60,\"percent\":1.0}")
//        
//        let sendProgress:(Float,ASActionInfoModel)->Void = { [weak self] progress,infoModel in
//            guard let self = self else { return }
//            infoModel.percent = progress
//            infoModel.actionStatus = .running
//            infoModel.status = "running"
//            self.replaceSaveInfoModel(model: infoModel)
//            self.stateDatauPblished = (.progress(progress,"Generating".localized + " \(Int(progress*100))%"),infoModel)
//        }
//        
//        var currentSec:Double = 0.0
//        guard let infoModel = infoModel else { return  }
//        infoModel.id = oldModel.id
//        currentSec+=1.0
//        kDelayOnMainThread(currentSec) {
//            self.stateDatauPblished = (.pending,nil) //通知首页进行更新
//        }
//        complete(true)
//        let multiple = 3.0
//        for index in 1...10 {
//            kDelayOnMainThread(currentSec+Double(index)*multiple) {
//                sendProgress(Float(index)/10.0,infoModel)
//            }
//        }
//
//        currentSec+=10.0*multiple+2.0
//        kDelayOnMainThread(currentSec) {
////            if kRandomBool() {
//                infoModel.actionStatus = .success
//                infoModel.status = "success"
//                self.replaceSaveInfoModel(model: infoModel)
//                self.stateDatauPblished = (.success(nil),infoModel)
//                self.handleGenerateSuccess()
////            }else{
////                infoModel.actionStatus = .failed
////                infoModel.status = "failed"
////                self.replaceSaveInfoModel(model: infoModel)
////                self.stateDatauPblished = (.failed("模拟失败模拟失败模拟失败模拟失败模拟失败模拟失败模拟失败模拟失败模拟失败模拟失败模拟失败模拟失败模拟失败模拟失败模拟失败模拟失败模拟失败模拟失败模拟失败模拟失败", 0),infoModel)
////            }
//        }
//    }
    
    func creatRing(oldModel:ASActionInfoModel,complete:@escaping (Bool)->Void) {
        
        initializeActionInfoModel(oldModel: oldModel)
        
        generatingProgress = 0
        stopNetwork = false
        
        
        currentActionInfoModel.status = "running"
        currentActionInfoModel.actionStatus = .running
        currentActionInfoModel.percent = 0
        replaceSaveInfoModel(model: currentActionInfoModel)
        
        //stateDatauPblished = (.progress(0.0,generating(progress: 0.0)),currentActionInfoModel)

        
        let request = currentActionInfoModel.request
        var prompt = request.prompt
        let inputText = request.inputText
        
        if inputText.count>0{
            if prompt.count > 0 {
                prompt = prompt + ", " + inputText
            }else {
                prompt = inputText
            }
        }
        
        var parameters:[String:Any] = ["prompt":prompt,
                                       "duration":request.duration,
                                       "device":ASDeviceInfo.systemMetadata
                                       ]
        if request.uploadUrl.count > 0 {
            parameters["audioUrl"] = request.uploadUrl
        }
        
        creatRequest = ASNetworkShared.executePOST(endpoint: .generateAudio,params:parameters) { [weak self] data,error in
            guard let self = self else { return }
            if stopNetwork == true { return }
            
            if let error = error {
                handleFailInfoModel(errorString: error.xhLocalizedMessage,code: error.xhStatusCode)
                complete(false)
            }else{
                if let dataDict = kNetWorkCodeSuccess(data: data),
                   let actionId = dataDict["actionId"] as? Int{
                    if stopNetwork == false {
                        complete(true)
                        self.stateDatauPblished = (.pending,nil) //通知首页进行更新
                        self.getActionInfo(action_id:actionId)
                    }
                }else{
                    handleFailInfoModel(errorString: "",code: 0)
                    complete(false)
                }
            }
        }
    }
    
    override func generating(progress: Float) -> String {
        let progress = Float(progress)*(0.9) // 预留 10% 进度给图片下载
        //Generating 0%-100%
        var progressInt = Int(progress*100)

        if progressInt > 99 {
            progressInt = 99
        }
        
        generatingProgress = progressInt
        return "Generating".localized + " \(progressInt)%"
    }


    override func cancelCleanContent() {
        super.cancelCleanContent()
        logPrint("cancelCleanContent")
        creatRequest?.cancel()
    }
}
